@@ -7,7 +7,7 @@
  *	remain fixed across releases, and they may not work, or may
  *	provide different semantics at some point in the future.
  *
- * Copyright (c) 2012-2020 Red Hat.
+ * Copyright (c) 2012-2021 Red Hat.
  * Copyright (c) 2008-2009 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
  *
@@ -24,13 +24,6 @@
 #ifndef PCP_LIBPCP_H
 #define PCP_LIBPCP_H
 
-/*
- * TODO
- * demote names from pm... to __pm... for
- * - pmHostSpec
- * - pmTimeZone
- */
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -209,7 +202,9 @@ PCP_CALL extern int __pmHasPMNSFileChanged(const char *);
 #define PDU_AUTH		PDU_ATTR
 #define PDU_LABEL_REQ		0x7012
 #define PDU_LABEL		0x7013
-#define PDU_FINISH		0x7013
+#define PDU_HIGHRES_FETCH	0x7014
+#define PDU_HIGHRES_RESULT	0x7015
+#define PDU_FINISH		0x7015
 #define PDU_MAX		 	(PDU_FINISH - PDU_START)
 
 typedef __uint32_t	__pmPDU;
@@ -284,6 +279,7 @@ typedef struct {
 #define PDU_FLAG_CERT_REQD	(1U<<7)
 #define PDU_FLAG_BAD_LABEL	(1U<<8)	/* bad, encoding issues */
 #define PDU_FLAG_LABELS		(1U<<9)
+#define PDU_FLAG_HIGHRES	(1U<<10)
 /* Credential CVERSION PDU elements look like this */
 typedef struct {
 #ifdef HAVE_BITFIELDS_LTOR
@@ -361,17 +357,22 @@ PCP_CALL extern int __pmSendXtendError(int, int, int, int);
 PCP_CALL extern int __pmDecodeXtendError(__pmPDU *, int *, int *);
 PCP_CALL extern int __pmSendResult(int, int, const pmResult *);
 PCP_CALL extern int __pmEncodeResult(int, const pmResult *, __pmPDU **);
+PCP_CALL extern int __pmSendHighResResult(int, int, const pmHighResResult *);
+PCP_CALL extern int __pmEncodeHighResResult(const pmHighResResult *, __pmPDU **);
 PCP_CALL extern int __pmDecodeResult(__pmPDU *, pmResult **);
+PCP_CALL extern int __pmDecodeHighResResult(__pmPDU *, pmHighResResult **);
 PCP_CALL extern int __pmSendProfile(int, int, int, pmProfile *);
 PCP_CALL extern int __pmDecodeProfile(__pmPDU *, int *, pmProfile **);
-PCP_CALL extern int __pmSendFetch(int, int, int, pmTimeval *, int, pmID *);
-PCP_CALL extern int __pmDecodeFetch(__pmPDU *, int *, pmTimeval *, int *, pmID **);
+PCP_CALL extern int __pmSendFetch(int, int, int, void *, int, pmID *);
+PCP_CALL extern int __pmDecodeFetch(__pmPDU *, int *, void *, int *, pmID **);
+PCP_CALL extern int __pmSendHighResFetch(int, int, int, int, pmID *);
+PCP_CALL extern int __pmDecodeHighResFetch(__pmPDU *, int *, int *, pmID **);
 PCP_CALL extern int __pmSendDescReq(int, int, pmID);
 PCP_CALL extern int __pmDecodeDescReq(__pmPDU *, pmID *);
 PCP_CALL extern int __pmSendDesc(int, int, pmDesc *);
 PCP_CALL extern int __pmDecodeDesc(__pmPDU *, pmDesc *);
-PCP_CALL extern int __pmSendInstanceReq(int, int, const pmTimeval *, pmInDom, int, const char *);
-PCP_CALL extern int __pmDecodeInstanceReq(__pmPDU *, pmTimeval *, pmInDom *, int *, char **);
+PCP_CALL extern int __pmSendInstanceReq(int, int, const void *, pmInDom, int, const char *);
+PCP_CALL extern int __pmDecodeInstanceReq(__pmPDU *, void *, pmInDom *, int *, char **);
 PCP_CALL extern int __pmSendInstance(int, int, pmInResult *);
 PCP_CALL extern int __pmDecodeInstance(__pmPDU *, pmInResult **);
 PCP_CALL extern int __pmSendTextReq(int, int, int, int);
@@ -513,9 +514,9 @@ typedef struct {
     char	*name;			/* hostname (always valid) */
     int		*ports;			/* array of host port numbers */
     int		nports;			/* number of ports in host port array */
-} pmHostSpec;
-PCP_CALL extern int __pmParseHostSpec(const char *, pmHostSpec **, int *, char **);
-PCP_CALL extern int __pmUnparseHostSpec(pmHostSpec *, int, char *, size_t);
+} __pmHostSpec;
+PCP_CALL extern int __pmParseHostSpec(const char *, __pmHostSpec **, int *, char **);
+PCP_CALL extern int __pmUnparseHostSpec(__pmHostSpec *, int, char *, size_t);
 
 /*
  * unfortunately, in this version, PCP archives are limited to no
@@ -592,8 +593,8 @@ PCP_CALL extern int __pmCompressedFileIndex(char *, size_t);
 typedef struct {
     int			pc_fd;		/* socket for comm with pmcd */
 					/* ... -1 means no connection */
-    pmHostSpec		*pc_hosts;	/* pmcd and proxy host specifications */
-    int			pc_nhosts;	/* number of pmHostSpec entries */
+    __pmHostSpec	*pc_hosts;	/* pmcd and proxy host specifications */
+    int			pc_nhosts;	/* number of hostspec entries */
     int			pc_timeout;	/* set if connect times out */
     int			pc_tout_sec;	/* timeout for __pmGetPDU */
     time_t		pc_again;	/* time to try again */
@@ -919,7 +920,9 @@ PCP_CALL extern __pmContext *__pmHandleToPtr(int);
 /* pmFetch helper routines, hooks for derivations and local contexts */
 PCP_CALL extern int __pmPrepareFetch(__pmContext *, int, const pmID *, pmID **);
 PCP_CALL extern int __pmFinishResult(__pmContext *, int, pmResult **);
+PCP_CALL extern int __pmFinishHighResResult(__pmContext *, int, pmHighResResult **);
 PCP_CALL extern int __pmFetchLocal(__pmContext *, int, pmID *, pmResult **);
+PCP_CALL extern int __pmHighResFetchLocal(__pmContext *, int, pmID *, pmHighResResult **);
 
 /* safely insert an atom value into a pmValue */
 PCP_CALL extern int __pmStuffValue(const pmAtomValue *, pmValue *, int);
@@ -1210,13 +1213,6 @@ PCP_CALL extern void __pmSetLocalContextFlag(pmOptions *);
 PCP_CALL extern void __pmSetLocalContextTable(pmOptions *, char *);
 PCP_CALL extern void __pmEndOptions(pmOptions *);
 
-/* real pmTimeZone declaration */
-typedef struct {
-    char		*label;		/* label to name tz */
-    char		*tz;		/* env $TZ */
-    int			handle;		/* handle from pmNewZone() */
-} pmTimeZone;
-
 /* work out local timezone */
 PCP_CALL extern char *__pmTimezone(void);		/* NOT thread-safe */
 PCP_CALL extern char *__pmTimezone_r(char *, int);
@@ -1302,9 +1298,9 @@ typedef enum {
 } __pmAttrKey;
 PCP_CALL extern __pmAttrKey __pmLookupAttrKey(const char *, size_t);
 PCP_CALL extern int __pmParseHostAttrsSpec(
-    const char *, pmHostSpec **, int *, __pmHashCtl *, char **);
+    const char *, __pmHostSpec **, int *, __pmHashCtl *, char **);
 PCP_CALL extern int __pmUnparseHostAttrsSpec(
-    pmHostSpec *, int, __pmHashCtl *, char *, size_t);
+    __pmHostSpec *, int, __pmHashCtl *, char *, size_t);
 
 /* SSL/TLS/IPv6 support via NSS/NSPR */
 PCP_CALL extern int __pmSecureServerCertificateSetup(const char *, const char *, const char *);
@@ -1323,8 +1319,8 @@ PCP_CALL extern int __pmInProfile(pmInDom, const pmProfile *, int);
 
 /* free malloc'd data structures */
 PCP_CALL extern void __pmFreeAttrsSpec(__pmHashCtl *);
-PCP_CALL extern void __pmFreeHostAttrsSpec(pmHostSpec *, int, __pmHashCtl *);
-PCP_CALL extern void __pmFreeHostSpec(pmHostSpec *, int);
+PCP_CALL extern void __pmFreeHostAttrsSpec(__pmHostSpec *, int, __pmHashCtl *);
+PCP_CALL extern void __pmFreeHostSpec(__pmHostSpec *, int);
 PCP_CALL extern void __pmFreeInResult(pmInResult *);
 PCP_CALL extern void __pmFreePMNS(__pmnsTree *);
 PCP_CALL extern void __pmFreeProfile(pmProfile *);
