@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2019 Red Hat.
+ * Copyright (c) 2012-2021 Red Hat.
  * Copyright (c) 1997,2004 Silicon Graphics, Inc.  All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or modify it
@@ -263,7 +263,7 @@ PCP_CALL extern int pmTrimNameSpace(void);
 /*
  * Expand a list of names to a list of metrics ids
  */
-PCP_CALL extern int pmLookupName(int, char **, pmID *);
+PCP_CALL extern int pmLookupName(int, const char **, pmID *);
 
 /*
  * Find the names of descendent nodes in the PMNS
@@ -324,6 +324,7 @@ PCP_CALL extern int pmGetInDomArchive(pmInDom, int **, char ***);
  * Given context ID, return the host name associated with that context,
  * or the empty string if no name can be found
  */
+PCP_CALL extern int pmGetHostName(int, char *, int);
 PCP_CALL extern const char *pmGetContextHostName(int);
 PCP_CALL extern char *pmGetContextHostName_r(int, char *, int);
 
@@ -495,7 +496,10 @@ typedef struct pmResult {
     pmValueSet		*vset[1];	/* set of value sets, one per PMID */
 } pmResult;
 
-/* High resolution event timer result from pmUnpackHighResEventRecords() */
+/*
+ * Result returned by pmHighResFetch() and high resolution event timer
+ * result from pmUnpackHighResEventRecords()
+ */
 typedef struct pmHighResResult {
     struct timespec	timestamp;	/* time stamped by collector */
     int			numpmid;	/* number of PMIDs */
@@ -524,9 +528,10 @@ typedef union {
  * the number of metrics in the agument.  
  */
 PCP_CALL extern int pmFetch(int, pmID *, pmResult **);
+PCP_CALL extern int pmHighResFetch(int, pmID *, pmHighResResult **);
 
 /*
- * PMCD state changes returned as pmFetch function result for PM_CONTEXT_HOST
+ * PMCD state changes returned as fetch function results for PM_CONTEXT_HOST
  * contexts, i.e. when communicating with PMCD
  */
 #define PMCD_ADD_AGENT		(1<<0)
@@ -655,6 +660,7 @@ PCP_CALL extern int pmGetArchiveEnd(struct timeval *);
 
 /* Free result buffer */
 PCP_CALL extern void pmFreeResult(pmResult *);
+PCP_CALL extern void pmFreeHighResResult(pmHighResResult *);
 
 /* Value extract from pmValue and type conversion */
 PCP_CALL extern int pmExtractValue(int, const pmValue *, int, pmAtomValue *, int);
@@ -770,6 +776,28 @@ PCP_CALL extern int pmflush(void);
  */
 PCP_CALL extern int pmsprintf(char *, size_t, const char *, ...) __PM_PRINTFLIKE(3,4);
 
+/*
+ * Safe version of fscanf("...%s...", buf) ... buf dynamically allocated,
+ * guaranteed to be null-byte terminated and returns strlen(buf)
+ */
+PCP_CALL extern size_t pmfstring(FILE *f, char **);
+
+/*
+ * Safe version of strncpy() ... args are deliberately different to
+ * strncpy() to guard against accidential misuse ... length is length
+ * of dest, not src ... also result is 0/-1 for success/truncation
+ * rather than the useless (!) char * = dest
+ */
+PCP_CALL extern int pmstrncpy(char *, size_t, const char *);
+
+/*
+ * Safe version of strncat() ... args are deliberately different to
+ * strncat() to guard against accidential misuse ... length is length
+ * of dest, not src ... also result is 0/-1 for success/truncation
+ * rather than the useless (!) char * = dest
+ */
+PCP_CALL extern int pmstrncat(char *, size_t, const char *);
+
 /*
  * Wrapper for config/environment variables. Warning: this will exit() with
  * a FATAL error if /etc/pcp.conf does not exist and $PCP_CONF is not set.
@@ -987,6 +1015,19 @@ PCP_CALL extern int pmLoadDerivedConfig(const char *);
 PCP_CALL extern int pmRegisterDerivedMetric(const char *, const char *, char **);
 PCP_CALL extern char *pmRegisterDerived(const char *, const char *);
 PCP_CALL extern char *pmDerivedErrStr(void);
+PCP_CALL extern int pmAddDerivedMetric(const char *, const char *, char **);
+PCP_CALL extern char *pmAddDerived(const char *, const char *);
+PCP_CALL extern int pmGetDerivedControl(int, int *);
+PCP_CALL extern int pmSetDerivedControl(int, int);
+
+/*
+ * pm{Get,Set}DerivedControl() information types ...
+ */
+#define PCP_DERIVED_GLOBAL_LIMIT	1
+#define PCP_DERIVED_CONTEXT_LIMIT	2
+#define PCP_DERIVED_DEBUG_SYNTAX	3
+#define PCP_DERIVED_DEBUG_SEMANTICS	4
+#define PCP_DERIVED_DEBUG_EVAL		5
 
 /*
  * Event Record support
@@ -1012,7 +1053,7 @@ typedef struct pmEventRecord {
 } pmEventRecord;
 
 typedef struct pmHighResEventRecord {
-    pmTimespec	er_timestamp;	/* must be 2 x 64-bit format */
+    pmTimespec		er_timestamp;	/* must be 2 x 64-bit format */
     unsigned int	er_flags;	/* event record characteristics */
     int			er_nparams;	/* number of er_param[] entries */
     pmEventParameter	er_param[1];
@@ -1099,6 +1140,7 @@ PCP_CALL extern int pmClearDebug(const char *);
  * New style ...
  * Note that comments are important ... these are extracted and
  * built into pmdbg.h.
+ * For the "add a new debug flag" recipe, see ../../libpcp/src/mk.pmdbg
  */
 typedef struct {
     int	pdu;		/* PDU traffic at the Xmit and Recv level */
@@ -1137,9 +1179,18 @@ typedef struct {
     int series;		/* Time series tracing */
     int	libweb;		/* Trace services from libpcp_web */
     int	alloc;		/* Miscellaneous alloc/free operations */
+    int	services;	/* Services and daemons */
     int	appl3;		/* Application-specific flag 3 */
     int	appl4;		/* Application-specific flag 4 */
     int	appl5;		/* Application-specific flag 5 */
+    int access;		/* Access controls */
+    int qa;		/* QA (transient, developers only) */
+    int search;		/* Text search tracing */
+    int query;		/* libpcp_web query parsing and evaulation */
+    int	compress;	/* Archive compress/decompress operations */
+    int dev0;		/* Developer flag 0 */
+    int dev1;		/* Developer flag 1 */
+    int dev2;		/* Developer flag 2 */
 } pmdebugoptions_t;
 
 PCP_DATA extern pmdebugoptions_t	pmDebugOptions;
@@ -1211,6 +1262,12 @@ PCP_CALL extern double pmtimevalSub(const struct timeval *, const struct timeval
 PCP_CALL extern double pmtimevalToReal(const struct timeval *);
 PCP_CALL extern void pmtimevalFromReal(double, struct timeval *);
 PCP_CALL extern void pmPrintStamp(FILE *, const struct timeval *);
+
+/* struct timespec manipulations */
+PCP_CALL extern int pmtimespecNow(struct timespec *);
+PCP_CALL extern void pmtimespecDec(struct timespec *, const struct timespec *);
+PCP_CALL extern double pmtimespecSub(const struct timespec *, const struct timespec *);
+PCP_CALL extern double pmtimespecToReal(const struct timespec *);
 PCP_CALL extern void pmPrintHighResStamp(FILE *, const struct timespec *);
 
 /* filesystem path name separator */
