@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2019 Red Hat.
+ * Copyright (c) 2017-2020 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -69,9 +69,14 @@ typedef enum pmSeriesFlags {
     PM_SERIES_FLAG_NONE		= (0),
     PM_SERIES_FLAG_METADATA	= (1 << 0),	/* only load metric metadata */
     PM_SERIES_FLAG_ACTIVE	= (1 << 1),	/* continual source updates */
+    PM_SERIES_FLAG_TEXT		= (1 << 2),	/* load metric & indom help */
     PM_SERIES_FLAG_ALL		= ((unsigned int)~PM_SERIES_FLAG_NONE)
 } pmSeriesFlags;
 
+typedef struct pmSeriesExpr {
+    sds		query;		/* canonical expression string */
+} pmSeriesExpr;
+
 typedef struct pmSeriesDesc {
     sds		indom;		/* dotted-pair instance domain identifier */
     sds		pmid;		/* dotted-triple metric identifier */
@@ -115,6 +120,7 @@ typedef void (*pmSeriesSetupCallBack)(void *);
 typedef int (*pmSeriesMatchCallBack)(pmSID, void *);
 typedef int (*pmSeriesStringCallBack)(pmSID, sds, void *);
 typedef int (*pmSeriesDescCallBack)(pmSID, pmSeriesDesc *, void *);
+typedef int (*pmSeriesExprCallBack)(pmSID, pmSeriesExpr *, void *);
 typedef int (*pmSeriesInstCallBack)(pmSID, pmSeriesInst *, void *);
 typedef int (*pmSeriesValueCallBack)(pmSID, pmSeriesValue *, void *);
 typedef int (*pmSeriesLabelCallBack)(pmSID, pmSeriesLabel *, void *);
@@ -123,6 +129,7 @@ typedef void (*pmSeriesDoneCallBack)(int, void *);
 typedef struct pmSeriesCallBacks {
     pmSeriesMatchCallBack	on_match;	/* one series identifier */
     pmSeriesDescCallBack	on_desc;	/* metric descriptor */
+    pmSeriesExprCallBack	on_expr;	/* query expression */
     pmSeriesInstCallBack	on_inst;	/* instance details */
     pmSeriesLabelCallBack	on_labelmap;	/* label name value pair */
     pmSeriesStringCallBack	on_instance;	/* one instance name */
@@ -152,6 +159,7 @@ extern int pmSeriesSetMetricRegistry(pmSeriesModule *, struct mmv_registry *);
 extern void pmSeriesClose(pmSeriesModule *);
 
 extern int pmSeriesDescs(pmSeriesSettings *, int, sds *, void *);
+extern int pmSeriesExprs(pmSeriesSettings *, int, sds *, void *);
 extern int pmSeriesLabels(pmSeriesSettings *, int, sds *, void *);
 extern int pmSeriesLabelValues(pmSeriesSettings *, int, sds *, void *);
 extern int pmSeriesInstances(pmSeriesSettings *, int, sds *, void *);
@@ -161,6 +169,12 @@ extern int pmSeriesValues(pmSeriesSettings *, pmSeriesTimeWindow *, int, sds *,
 extern int pmSeriesQuery(pmSeriesSettings *, sds, pmSeriesFlags, void *);
 extern int pmSeriesLoad(pmSeriesSettings *, sds, pmSeriesFlags, void *);
 
+/* libpcp_web timer list interface - global, thread-safe */
+typedef void (*pmWebTimerCallBack)(void *);
+extern int pmWebTimerSetMetricRegistry(struct mmv_registry *); /* for generic server metrics */
+extern int pmWebTimerRegister(pmWebTimerCallBack, void *);
+extern int pmWebTimerRelease(int); /* deregister one timer */
+
 /*
  * Asynchronous archive location and contents discovery services
  */
@@ -207,6 +221,7 @@ typedef struct pmDiscoverCallBacks {
     pmDiscoverValuesCallBack	on_values;	/* metrics value set arrived */
     pmDiscoverInDomCallBack	on_indom;	/* instance domain discovered */
     pmDiscoverTextCallBack	on_text;	/* new help text discovered */
+    struct pmDiscoverCallBacks	*next;		/* optional list of callbacks */
 } pmDiscoverCallBacks;
 
 typedef struct pmDiscoverSettings {
@@ -354,6 +369,7 @@ extern void pmWebGroupChildren(pmWebGroupSettings *, sds, struct dict *, void *)
 extern void pmWebGroupProfile(pmWebGroupSettings *, sds, struct dict *, void *);
 extern void pmWebGroupScrape(pmWebGroupSettings *, sds, struct dict *, void *);
 extern void pmWebGroupStore(pmWebGroupSettings *, sds, struct dict *, void *);
+extern void pmWebGroupDestroy(pmWebGroupSettings *, sds, void *);
 
 extern int pmWebGroupSetup(pmWebGroupModule *);
 extern int pmWebGroupSetEventLoop(pmWebGroupModule *, void *);
@@ -361,6 +377,113 @@ extern int pmWebGroupSetConfiguration(pmWebGroupModule *, struct dict *);
 extern int pmWebGroupSetMetricRegistry(pmWebGroupModule *, struct mmv_registry *);
 extern void pmWebGroupClose(pmWebGroupModule *);
 
+/*
+ * Full text search for metrics and instance domains.
+ */
+
+typedef enum pmSearchFlags {
+    PM_SEARCH_FLAG_NONE		= (0),
+    PM_SEARCH_FLAG_NOTEXT	= (1 << 0),	/* no text in the response */
+    PM_SEARCH_FLAG_HIGHLIGHT	= (1 << 1),	/* highlight search terms */
+} pmSearchFlags;
+
+typedef enum pmSearchTextType {
+    PM_SEARCH_TYPE_UNKNOWN	= 0,
+    PM_SEARCH_TYPE_METRIC	= 1,
+    PM_SEARCH_TYPE_INDOM	= 2,
+    PM_SEARCH_TYPE_INST		= 3
+} pmSearchTextType;
+
+typedef struct pmSearchTextRequest {
+    sds			query;		/* query string */
+    pmSearchFlags	flags;		/* query control bits */
+    unsigned int	count;		/* maximum results to return */
+    unsigned int	offset;		/* results pagination offset */
+
+    unsigned int	type_metric : 1;	/* restrict query types */
+    unsigned int	type_indom : 1;
+    unsigned int	type_inst : 1;
+    unsigned int	type_pad : 1;
+
+    unsigned int	highlight_name : 1;	/* highlight results */
+    unsigned int	highlight_oneline : 1;
+    unsigned int	highlight_helptext : 1;
+
+    unsigned int	infields_name : 1;	/* restrict query fields */
+    unsigned int	infields_oneline : 1;
+    unsigned int	infields_helptext : 1;
+
+    unsigned int	return_name : 1;	/* restrict returned fields */
+    unsigned int	return_indom : 1;
+    unsigned int	return_oneline : 1;
+    unsigned int	return_helptext : 1;
+    unsigned int	return_type : 1;
+
+    unsigned int	reserved: 17;	/* zero padding */
+} pmSearchTextRequest;
+
+typedef struct pmSearchTextResult {
+    unsigned int	total;		/* total number of results */
+    unsigned int	count;		/* query result index 'count' */
+    double		timer;		/* elapsed time (in seconds) */
+    double		score;		/* search engine hit ranking */
+
+    pmSearchTextType	type;		/* query result document type */
+    sds			docid;		/* unique result identifier */
+    sds			name;		/* metric / instance name */
+    sds			indom;
+    sds			oneline;
+    sds			helptext;
+} pmSearchTextResult;
+
+typedef struct pmSearchMetrics {
+    unsigned long long	docs;		/* number of documents */
+    unsigned long long	terms;		/* number of distinct terms */
+    unsigned long long	records;	/* number of search records */
+    double		inverted_sz_mb;
+    double		inverted_cap_mb;
+    double		inverted_cap_ovh;
+    double		offset_vectors_sz_mb;
+    double		skip_index_size_mb;
+    double		score_index_size_mb;
+    double		records_per_doc_avg;
+    double		bytes_per_record_avg;
+    double		offsets_per_term_avg;
+    double		offset_bits_per_record_avg;
+} pmSearchMetrics;
+
+typedef void (*pmSearchSetupCallBack)(void *);
+typedef void (*pmSearchTextResultCallBack)(pmSearchTextResult *, void *);
+typedef void (*pmSearchMetricsCallBack)(pmSearchMetrics *, void *);
+typedef void (*pmSearchDoneCallBack)(int, void *);
+
+typedef struct pmSearchCallBacks {
+    pmSearchTextResultCallBack	on_text_result;	/* text search hit */
+    pmSearchMetricsCallBack	on_metrics;	/* runtime stats */
+    pmSearchDoneCallBack	on_done;	/* request completed */
+} pmSearchCallBacks;
+
+typedef struct pmSeriesModule pmSearchModule;	/* shared structure */
+
+typedef struct pmSearchSettings {
+    pmSearchModule		module;
+    pmSearchCallBacks		callbacks;
+} pmSearchSettings;
+
+extern int pmSearchSetup(pmSearchModule *, void *);
+extern int pmSearchSetSlots(pmSearchModule *, void *);
+extern int pmSearchSetEventLoop(pmSearchModule *, void *);
+extern int pmSearchSetConfiguration(pmSearchModule *, struct dict *);
+extern int pmSearchSetMetricRegistry(pmSearchModule *, struct mmv_registry *);
+extern void pmSearchClose(pmSearchModule *);
+extern int pmSearchEnabled(void *);
+
+extern const char *pmSearchTextTypeStr(pmSearchTextType);
+extern int pmSearchInfo(pmSearchSettings *, sds, void *);
+extern int pmSearchTextInDom(pmSearchSettings *, pmSearchTextRequest *, void *);
+extern int pmSearchTextSuggest(pmSearchSettings *, pmSearchTextRequest *, void *);
+extern int pmSearchTextQuery(pmSearchSettings *, pmSearchTextRequest *, void *);
+
 #ifdef __cplusplus
 }
 #endif
