@@ -233,7 +233,7 @@ int
 pmGetHostName(int handle, char *buf, int buflen)
 {
     __pmContext *ctxp;
-    char	*name;
+    const char	*name;
     pmID	pmid;
     pmResult	*resp;
     int		save_handle;
@@ -974,23 +974,21 @@ initarchive(__pmContext	*ctxp, const char *name)
     }
     if (namelist)
 	free(namelist);
-    if (acp) {
-	if (acp->ac_log_list) {
-	    while (acp->ac_num_logs > 0) {
-		--acp->ac_num_logs;
-		if (acp->ac_log_list[acp->ac_num_logs]) {
-		    free(acp->ac_log_list[acp->ac_num_logs]->ml_name);
-		    free(acp->ac_log_list[acp->ac_num_logs]->ml_hostname);
-		    free(acp->ac_log_list[acp->ac_num_logs]->ml_tz);
-		    free(acp->ac_log_list[acp->ac_num_logs]);
-		}
+    if (acp->ac_log_list) {
+	while (acp->ac_num_logs > 0) {
+	    --acp->ac_num_logs;
+	    if (acp->ac_log_list[acp->ac_num_logs]) {
+		free(acp->ac_log_list[acp->ac_num_logs]->ml_name);
+		free(acp->ac_log_list[acp->ac_num_logs]->ml_hostname);
+		free(acp->ac_log_list[acp->ac_num_logs]->ml_tz);
+		free(acp->ac_log_list[acp->ac_num_logs]);
 	    }
-	    free(acp->ac_log_list);
 	}
-	if (acp->ac_log && --acp->ac_log->l_refcnt == 0)
-	    free(acp->ac_log);
-	free(acp);
+	free(acp->ac_log_list);
     }
+    if (acp->ac_log && --acp->ac_log->l_refcnt == 0)
+	free(acp->ac_log);
+    free(acp);
     ctxp->c_archctl = NULL;
     return sts;
 }
@@ -1116,7 +1114,7 @@ INIT_CONTEXT:
 
     if (new->c_type == PM_CONTEXT_HOST) {
 	__pmHashCtl	*attrs = &new->c_attrs;
-	pmHostSpec	*hosts = NULL;
+	__pmHostSpec	*hosts = NULL;
 	int		nhosts;
 	char		*errmsg;
 
@@ -1285,8 +1283,8 @@ pmReconnectContext(int handle)
     }
 
     ctxp = contexts[ctxnum];
-    PM_LOCK(ctxp->c_lock);
     PM_UNLOCK(contexts_lock);
+    PM_LOCK(ctxp->c_lock);
     ctl = ctxp->c_pmcd;
     if (ctxp->c_type == PM_CONTEXT_HOST) {
 	if (ctl->pc_timeout && time(NULL) < ctl->pc_again) {
@@ -1307,11 +1305,11 @@ pmReconnectContext(int handle)
 
 	if ((sts = __pmConnectPMCD(ctl->pc_hosts, ctl->pc_nhosts,
 				   ctxp->c_flags, &ctxp->c_attrs)) < 0) {
+	    PM_UNLOCK(ctxp->c_lock);
 	    waitawhile(ctl);
 	    if (pmDebugOptions.context)
 		fprintf(stderr, "pmReconnectContext(%d), failed (wait %d secs before next attempt)\n",
 		    handle, (int)(ctl->pc_again - time(NULL)));
-	    PM_UNLOCK(ctxp->c_lock);
 	    sts = -ETIMEDOUT;
 	    goto pmapi_return;
 	}
