@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2019 Red Hat.
+ * Copyright (c) 2012-2021 Red Hat.
  * Copyright (c) 1995-2001 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This library is free software; you can redistribute it and/or modify it
@@ -324,7 +324,7 @@ extern int __pmAddPorts(const char *, int **, int) _PCP_HIDDEN;
  * Internal variants for with extra __pmContext parameter to allow
  * for context being locked or not locked
  */
-extern int pmLookupName_ctx(__pmContext *, int, int, char **, pmID *) _PCP_HIDDEN;
+extern int pmLookupName_ctx(__pmContext *, int, int, const char **, pmID *) _PCP_HIDDEN;
 extern int pmNameAll_ctx(__pmContext *, pmID, char ***) _PCP_HIDDEN;
 extern int pmLookupDesc_ctx(__pmContext *, int, pmID, pmDesc *) _PCP_HIDDEN;
 extern int pmNameInDom_ctx(__pmContext *, pmInDom, int, char **) _PCP_HIDDEN;
@@ -333,8 +333,11 @@ extern int pmGetInDomArchive_ctx(__pmContext *, pmInDom, int **, char ***) _PCP_
 extern int pmFetch_ctx(__pmContext *, int, pmID *, pmResult **) _PCP_HIDDEN;
 extern int pmStore_ctx(__pmContext *, const pmResult *) _PCP_HIDDEN;
 extern int __pmDecodeResult_ctx(__pmContext *, __pmPDU *, pmResult **) _PCP_HIDDEN;
+extern int __pmDecodeHighResResult_ctx(__pmContext *, __pmPDU *, pmHighResResult **) _PCP_HIDDEN;
 extern int __pmSendResult_ctx(__pmContext *, int, int, const pmResult *) _PCP_HIDDEN;
+extern int __pmSendHighResResult_ctx(__pmContext *, int, int, const pmHighResResult *) _PCP_HIDDEN;
 extern void __pmDumpResult_ctx(__pmContext *, FILE *, const pmResult *) _PCP_HIDDEN;
+extern void __pmDumpHighResResult_ctx(__pmContext *, FILE *, const pmHighResResult *) _PCP_HIDDEN;
 extern int pmGetArchiveEnd_ctx(__pmContext *, struct timeval *) _PCP_HIDDEN;
 extern int __pmGetArchiveEnd_ctx(__pmContext *, struct timeval *) _PCP_HIDDEN;
 extern int __pmLogGenerateMark_ctx(__pmContext *, int, pmResult **) _PCP_HIDDEN;
@@ -387,14 +390,14 @@ extern void	     __pmSockAddrSetScope(__pmSockAddr *, int) _PCP_HIDDEN;
 extern __pmSockAddr *__pmSockAddrFirstSubnetAddr(const __pmSockAddr *, int) _PCP_HIDDEN;
 extern __pmSockAddr *__pmSockAddrNextSubnetAddr(__pmSockAddr *, int) _PCP_HIDDEN;
 
-extern int __pmConnectPMCD(pmHostSpec *, int, int, __pmHashCtl *) _PCP_HIDDEN;
+extern int __pmConnectPMCD(__pmHostSpec *, int, int, __pmHashCtl *) _PCP_HIDDEN;
 
 extern int __pmConnectLocal(__pmHashCtl *) _PCP_HIDDEN;
 extern int __pmAuxConnectPMCD(const char *) _PCP_HIDDEN;
 extern int __pmAuxConnectPMCDUnixSocket(const char *) _PCP_HIDDEN;
-extern int __pmAddHostPorts(pmHostSpec *, int *, int) _PCP_HIDDEN;
-extern void __pmDropHostPort(pmHostSpec *) _PCP_HIDDEN;
-extern void __pmConnectGetPorts(pmHostSpec *) _PCP_HIDDEN;
+extern int __pmAddHostPorts(__pmHostSpec *, int *, int) _PCP_HIDDEN;
+extern void __pmDropHostPort(__pmHostSpec *) _PCP_HIDDEN;
+extern void __pmConnectGetPorts(__pmHostSpec *) _PCP_HIDDEN;
 
 extern int __pmLogOpen(const char *, __pmContext *) _PCP_HIDDEN;
 extern int __pmLogReopen(const char *, __pmContext *) _PCP_HIDDEN;
@@ -404,7 +407,7 @@ extern int __pmLogGenerateMark(__pmLogCtl *, int, pmResult **) _PCP_HIDDEN;
 extern int __pmLogFetchInterp(__pmContext *, int, pmID *, pmResult **) _PCP_HIDDEN;
 extern int __pmGetArchiveLabel(__pmLogCtl *, pmLogLabel *) _PCP_HIDDEN;
 extern pmTimeval *__pmLogStartTime(__pmArchCtl *) _PCP_HIDDEN;
-extern void __pmLogSetTime(__pmContext *) _PCP_HIDDEN;
+extern int __pmLogSetTime(__pmContext *) _PCP_HIDDEN;
 extern void __pmLogResetInterp(__pmContext *) _PCP_HIDDEN;
 extern void __pmArchCtlFree(__pmArchCtl *) _PCP_HIDDEN;
 extern int __pmLogChangeArchive(__pmContext *, int) _PCP_HIDDEN;
