@@ -1003,7 +1003,7 @@ time_caliper(__pmContext *ctxp, instcntl_t *icp)
 			free(instp);
 			return;
 		    }
-		    if ((ip = __pmHashSearch((unsigned int)idp->instlist[j], &indomp->hashinst)) == NULL) {
+		    if (__pmHashSearch((unsigned int)idp->instlist[j], &indomp->hashinst) == NULL) {
 			char	strbuf[20];
 			fprintf(stderr, "time_caliper: Botch: indom %s inst %d hashinst __pmHashSearch failed\n", pmInDomStr_r(icp->metric->desc.indom, strbuf, sizeof(strbuf)), idp->instlist[j]);
 			return;
@@ -1312,7 +1312,9 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
     }
 
     /* get to the last remembered place */
-    __pmLogChangeVol(ctxp->c_archctl, ctxp->c_archctl->ac_vol);
+    sts = __pmLogChangeVol(ctxp->c_archctl, ctxp->c_archctl->ac_vol);
+    if (sts < 0)
+	goto all_done;
     __pmFseek(ctxp->c_archctl->ac_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
 
     seen_mark = 0;	/* interested in <mark> records seen from here on */
@@ -1397,7 +1399,9 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	 * at least one metric requires a bound from earlier in the log ...
 	 * position ourselves, ... and search
 	 */
-	__pmLogChangeVol(ctxp->c_archctl, ctxp->c_archctl->ac_vol);
+	sts = __pmLogChangeVol(ctxp->c_archctl, ctxp->c_archctl->ac_vol);
+	if (sts < 0)
+	    goto all_done;
 	__pmFseek(ctxp->c_archctl->ac_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
 	done = 0;
 
@@ -1542,7 +1546,9 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	 * at least one metric requires a bound from later in the log ...
 	 * position ourselves ... and search
 	 */
-	__pmLogChangeVol(ctxp->c_archctl, ctxp->c_archctl->ac_vol);
+	sts = __pmLogChangeVol(ctxp->c_archctl, ctxp->c_archctl->ac_vol);
+	if (sts < 0)
+	    goto all_done;
 	__pmFseek(ctxp->c_archctl->ac_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
 	done = 0;
 
