@@ -169,7 +169,7 @@ __pmCloseSocket(int fd)
 	}
     }
 
-    if (fd != -1) {
+    if (fd >= 0) {
 #if defined(IS_MINGW)
 	closesocket(fd);
 #else
@@ -1548,6 +1548,9 @@ __pmRead(int fd, void *buffer, size_t length)
 {
     __pmSecureSocket socket;
 
+    if (fd < 0)
+	return -EBADF;
+
     if (__pmDataIPC(fd, &socket) == 0 && socket.nsprFd) {
 	ssize_t size = PR_Read(socket.nsprFd, buffer, length);
 	if (size < 0)
@@ -1562,6 +1565,9 @@ __pmSend(int fd, const void *buffer, size_t length, int flags)
 {
     __pmSecureSocket socket;
 
+    if (fd < 0)
+	return -EBADF;
+
     if (__pmDataIPC(fd, &socket) == 0 && socket.nsprFd) {
 	ssize_t size = PR_Write(socket.nsprFd, buffer, length);
 	if (size < 0)
@@ -1577,6 +1583,9 @@ __pmRecv(int fd, void *buffer, size_t length, int flags)
     __pmSecureSocket	socket;
     ssize_t		size;
 
+    if (fd < 0)
+	return -EBADF;
+
     if (__pmDataIPC(fd, &socket) == 0 && socket.nsprFd) {
 	if (pmDebugOptions.pdu && pmDebugOptions.desperate) {
 	    fprintf(stderr, "%s:__pmRecv[secure](", __FILE__);
@@ -1615,6 +1624,9 @@ __pmSocketReady(int fd, struct timeval *timeout)
     __pmSecureSocket socket;
     __pmFdSet onefd;
 
+    if (fd < 0)
+	return -EBADF;
+
     if (__pmDataIPC(fd, &socket) == 0 && socket.sslFd)
         if (SSL_DataPending(socket.sslFd))
 	    return 1;	/* proceed without blocking */
