@@ -108,6 +108,9 @@ __pmSquashTZ(char *tzbuffer)
 
     PM_ASSERT_IS_LOCKED(__pmLock_extcall);
 
+    if (pmDebugOptions.context)
+	fprintf(stderr, "__pmSquashTZ(%s)", tzbuffer);
+
     tzset();
     t = localtime(&now);		/* THREADSAFE */
 
@@ -117,8 +120,8 @@ __pmSquashTZ(char *tzbuffer)
     {
 	char tzoffset[6]; /* +1200\0 */
 
-	strftime (tzoffset, sizeof (tzoffset), "%z", t);
-	offset = -strtol (tzoffset, NULL, 10);
+	strftime(tzoffset, sizeof (tzoffset), "%z", t);
+	offset = -strtol(tzoffset, NULL, 10);
 	offset = ((offset/100) * 3600) + ((offset%100) * 60);
     }
 #else
@@ -133,7 +136,7 @@ __pmSquashTZ(char *tzbuffer)
 
     if (offset != 0) {
 	int hours = offset / 3600;
-	int mins = abs ((int)((offset % 3600) / 60));
+	int mins = abs((int)((offset % 3600) / 60));
 	int len = (int)strlen(tzn);
 
 	if (mins == 0) {
@@ -151,6 +154,8 @@ __pmSquashTZ(char *tzbuffer)
 	strncpy(tzbuffer, tzn, PM_TZ_MAXLEN);
 	tzbuffer[PM_TZ_MAXLEN] = '\0';
     }
+    if (pmDebugOptions.context)
+	fprintf(stderr, " -> %s\n", tzbuffer);
     setenv("TZ", tzbuffer, 1);		/* THREADSAFE */
 
     return;
@@ -177,6 +182,9 @@ __pmSquashTZ(char *tzbuffer)
     wchar_t *src;
     div_t d;
 
+    if (pmDebugOptions.context)
+	fprintf(stderr, "__pmSquashTZ(%s)", tzbuffer);
+
     GetTimeZoneInformation(&tz);
     dst = cp = tzbuf;
     off = tzoff;
@@ -260,6 +268,8 @@ __pmSquashTZ(char *tzbuffer)
     tzn = tzname[(t->tm_isdst > 0)];
 
     pmsprintf(tzbuffer, PM_TZ_MAXLEN, "%s%s", tzn, tzoff);
+    if (pmDebugOptions.context)
+	fprintf(stderr, " -> %s\n", tzbuffer);
     setenv("TZ", tzbuffer, 1);		/* THREADSAFE */
 
     return;
@@ -293,14 +303,18 @@ __pmTimezone(void)
     }
 
     if (tz == NULL || tz[0] == ':') {
-	/* NO TZ in the environment - invent one. If TZ starts with a colon,
-	 * it's an Olson-style TZ and it does not supported on all IRIXes, so
-	 * squash it into a simple one (pv#788431). */
+	/*
+	 * NO TZ in the environment - invent one. If TZ starts with a colon,
+	 * it's a file-based TZ and it is not supported on all platforms, so
+	 * squash it into a simple one.
+	 */
 	__pmSquashTZ(tzbuffer);
 	tz = tzbuffer;
     } else if (strlen(tz) > PM_TZ_MAXLEN) {
-	/* TZ is too long to fit into the internal PCP timezone structs
-	 * let's try to sqash it a bit */
+	/*
+	 * TZ is too long to fit into the internal PCP timezone structs
+	 * let's try to squash it a bit.
+	 * */
 	char *tb;
 
 	if ((tb = strdup(tz)) == NULL) {
@@ -321,7 +335,7 @@ __pmTimezone(void)
 	    }
 
 	    if (strlen(tb) > PM_TZ_MAXLEN) { 
-		/* Still too long - let's pretend it's Olson */
+		/* Still too long - let's pretend it's file-based */
 		__pmSquashTZ(tzbuffer);
 		tz = tzbuffer;
 	    } else {
@@ -443,7 +457,7 @@ pmNewContextZone(void)
     }
     else {
 	/* assume PM_CONTEXT_HOST */
-	char		*name = "pmcd.timezone";
+	const char	*name = "pmcd.timezone";
 	pmID		pmid;
 	pmResult	*rp;
 
