@@ -281,6 +281,14 @@ logreopen(const char *progname, const char *logname, FILE *oldstream,
 
     fflush(oldstream);
     *status = 0;		/* set to one if all this works ... */
+
+    if (logname == NULL) {
+	/*
+	 * no logfile name, no dice
+	 */
+	return NULL;
+    }
+
     oldfd = fileno(oldstream);
     if ((dupoldfd = dup(oldfd)) >= 0) {
 	/*
@@ -555,7 +563,7 @@ pmEventFlagsStr_r(int flags, char *buf, int buflen)
 
     buf[0] = '\0';
     if (flags & PM_EVENT_FLAG_POINT) {
-	if (started++) strcat(buf, ",");
+	started++;
 	strcat(buf, "point");
     }
     if (flags & PM_EVENT_FLAG_START) {
@@ -1142,10 +1150,9 @@ void
 pmNoMem(const char *where, size_t size, int fatal)
 {
     char	errmsg[PM_MAXERRMSGLEN];
-    pmNotifyErr(fatal ? LOG_ERR : LOG_WARNING,
-			"%s: malloc(%d) failed: %s",
-			where, (int)size, osstrerror_r(errmsg, sizeof(errmsg)));
-    if (fatal)
+    pmNotifyErr(fatal ? LOG_ERR : LOG_WARNING, "%s: malloc(%zu) failed: %s",
+			where, size, osstrerror_r(errmsg, sizeof(errmsg)));
+    if (fatal == PM_FATAL_ERR)
 	exit(1);
 }
 
@@ -1193,6 +1200,12 @@ __pmGetTimespec(struct timespec *ts)
 #endif
 }
 
+int
+pmtimespecNow(struct timespec *ts)
+{
+    return __pmGetTimespec(ts);
+}
+
 /*
  * a : b for pmTimeval ... <0 for a<b, ==0 for a==b, >0 for a>b
  */
@@ -1657,6 +1670,7 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_ALLOC);
     }
 
     va_copy(save_arg, arg);
+    va_end(save_arg);
     avail = msgbuflen - msgsize - 1;
     for ( ; ; ) {
 	char	*msgbuf_tmp;
@@ -1665,6 +1679,7 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_ALLOC);
 	    if (bytes < avail)
 		break;
 	    va_copy(arg, save_arg);	/* will need to call vsnprintf() again */
+	    va_end(arg);
 	}
 	msgbuflen += MSGCHUNK;
 PM_FAULT_POINT("libpcp/" __FILE__ ":2", PM_FAULT_ALLOC);
@@ -1682,13 +1697,11 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":2", PM_FAULT_ALLOC);
 	    msgsize = 0;
 	    PM_UNLOCK(util_lock);
 	    pmNoMem("vmprintf realloc", msgbuflen_tmp, PM_RECOV_ERR);
-	    va_end(save_arg);
 	    return -ENOMEM;
 	}
 	msgbuf = msgbuf_tmp;
 	avail = msgbuflen - msgsize - 1;
     }
-    va_end(save_arg);
     msgsize += bytes;
 
     PM_UNLOCK(util_lock);
@@ -1803,32 +1816,6 @@ pmflush(void)
     return sts;
 }
 
-int
-pmsprintf(char *str, size_t size, const char *fmt, ...)
-{
-    va_list	arg;
-    int		bytes;
-
-    /* bad input given - treated as garbage in, garbage out */
-    if (size == 0)
-	return 0;
-
-    va_start(arg, fmt);
-    bytes = vsnprintf(str, size, fmt, arg);
-    va_end(arg);
-    if (bytes < size) {
-	if (bytes > 0)	/* usual case, null terminated here */
-	    return bytes;
-	/* safest option - treat all errors as empty string */
-	*str = '\0';
-	return 1;
-    }
-    /* ensure (truncated) string is always null terminated. */
-    bytes = size - 1;
-    str[bytes] = '\0';
-    return bytes;
-}
-
 /*
  * Set the pmcd client identity as exported by pmcd.client.whoami
  *
@@ -2040,7 +2027,11 @@ __pmMakePath(const char *dir, mode_t mode)
     for (p = path+1; *p != '\0'; p++) {
 	if (*p == pmPathSeparator()) {
 	    *p = '\0';
-	    mkdir2(path, mode);
+	    if (mkdir2(path, mode) < 0) {
+		/* dir may already exist */
+		if (oserror() != EEXIST)
+		    return -1;
+	    }
 	    *p = pmPathSeparator();
 	}
     }
