@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2020 Red Hat.
+ * Copyright (c) 2018-2021 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,11 +30,45 @@ static pmDiscoverCallBacks **discoverCallBackTable;
 static char *pmDiscoverFlagsStr(pmDiscover *);
 
 /* internal hash table of discovered paths */
-#define PM_DISCOVER_HASHTAB_SIZE 16
+#define PM_DISCOVER_HASHTAB_SIZE 32
 static pmDiscover *discover_hashtable[PM_DISCOVER_HASHTAB_SIZE];
 
-/* pmlogger_daily log-roll lock count */
-static int lockcnt = 0;
+/* number of archives or directories currently being monitored */
+static int n_monitored = 0;
+
+/* stats helpers */
+static void
+pmDiscoverStatsAdd(pmDiscoverModule *module, const char *name, const char *inst, double count)
+{
+    discoverModuleData	*data;
+
+    if (module != NULL && (data = getDiscoverModuleData(module)) != NULL)
+	mmv_stats_add(data->metrics_handle, name, inst, count);
+}
+
+static void
+pmDiscoverStatsSet(pmDiscoverModule *module, const char *name, const char *inst, double value)
+{
+    discoverModuleData	*data;
+
+    if (module != NULL && (data = getDiscoverModuleData(module)) != NULL)
+    	mmv_stats_set(data->metrics_handle, name, inst, value);
+}
+
+static int
+pmDiscoverGetInflightRedisRequests(pmDiscoverModule *module)
+{
+    discoverModuleData	*data;
+
+    if (module == NULL)
+        return 0;
+
+    data = getDiscoverModuleData(module);
+    if (data == NULL || data->slots == NULL)
+	return 0;
+
+    return data->slots->inflight_requests;
+}
 
 /* FNV string hash algorithm. Return unsigned in range 0 .. limit-1 */
 static unsigned int
@@ -99,6 +133,7 @@ pmDiscoverLookupAdd(const char *fullpath, pmDiscoverModule *module, void *arg)
 	    discover_hashtable[k] = h;
 	else
 	    p->next = h;
+	pmDiscoverStatsSet(h->module, "monitored", NULL, ++n_monitored);
 	if (pmDebugOptions.discovery)
 	    fprintf(stderr, "pmDiscoverLookupAdd: --> new entry %s\n", name);
 
@@ -388,22 +423,11 @@ is_deleted(pmDiscover *p, struct stat *sbuf)
     return ret;
 }
 
-static void
-logdir_is_locked_callBack(pmDiscover *p, void *arg)
-{
-    int			*cntp = (int *)arg;
-    char		sep = pmPathSeparator();
-    char		path[MAXNAMELEN];
-
-    pmsprintf(path, sizeof(path), "%s%c%s", p->context.name, sep, "lock");
-    if (access(path, F_OK) == 0)
-    	(*cntp)++;
-}
-
 static void
 check_deleted(pmDiscover *p)
 {
     struct stat sbuf;
+
     if (!(p->flags & PM_DISCOVER_FLAGS_DELETED) && is_deleted(p, &sbuf))
     	p->flags |= PM_DISCOVER_FLAGS_DELETED;
 }
@@ -416,37 +440,8 @@ fs_change_callBack(uv_fs_event_t *handle, const char *filename, int events, int
     pmDiscover		*p;
     char		*s;
     sds			path;
-    int			count = 0;
     struct stat		statbuf;
 
-    /*
-     * check if logs are currently being rolled by pmlogger_daily et al
-     * in any of the directories we are tracking. For mutex, the log control
-     * scripts use a 'lock' file in each directory as it is processed.
-     */
-    pmDiscoverTraverseArg(PM_DISCOVER_FLAGS_DIRECTORY,
-    	logdir_is_locked_callBack, (void *)&count);
-
-    if (lockcnt == 0 && count > 0) {
-	/* log-rolling has started */
-    	fprintf(stderr, "%s discovery callback: log-rolling in progress\n", stamp());
-	lockcnt = count;
-	return;
-    }
-
-    if (lockcnt > 0 && count > 0) {
-	/* log-rolling is still in progress */
-	lockcnt = count;
-	return;
-    }
-
-    if (lockcnt > 0 && count == 0) {
-    	/* log-rolling is finished: check what got deleted, and then purge */
-    	fprintf(stderr, "%s discovery callback: finished log-rolling\n", stamp());
-	pmDiscoverTraverse(PM_DISCOVER_FLAGS_META|PM_DISCOVER_FLAGS_DATAVOL, check_deleted);
-    }
-    lockcnt = count;
-
     uv_fs_event_getpath(handle, buffer, &bytes);
     path = sdsnewlen(buffer, bytes);
 
@@ -752,16 +747,19 @@ pmDiscoverInvokeMetricCallBacks(pmDiscover *p, pmTimespec *ts, pmDesc *desc,
 	__pmContext	*ctxp = __pmHandleToPtr(p->ctx);
 	__pmArchCtl	*acp = ctxp->c_archctl;
 	char		idstr[32];
+	char		errmsg[PM_MAXERRMSGLEN];
 
 	if ((sts = __pmLogAddDesc(acp, desc)) < 0)
-	    fprintf(stderr, "%s: failed to add metric descriptor for %s\n",
+	    fprintf(stderr, "%s: failed to add metric descriptor for %s: %s\n",
 			    "pmDiscoverInvokeMetricCallBacks",
-			    pmIDStr_r(desc->pmid, idstr, sizeof(idstr)));
+			    pmIDStr_r(desc->pmid, idstr, sizeof(idstr)),
+			    pmErrStr_r(sts, errmsg, sizeof(errmsg)));
 	for (i = 0; i < numnames; i++) {
 	    if ((sts = __pmLogAddPMNSNode(acp, desc->pmid, names[i])) < 0)
-		fprintf(stderr, "%s: failed to add metric name %s for %s\n",
+		fprintf(stderr, "%s: failed to add metric name %s for %s: %s\n",
 				"pmDiscoverInvokeMetricCallBacks", names[i],
-				pmIDStr_r(desc->pmid, idstr, sizeof(idstr)));
+				pmIDStr_r(desc->pmid, idstr, sizeof(idstr)),
+				pmErrStr_r(sts, errmsg, sizeof(errmsg)));
 	}
 	PM_UNLOCK(ctxp->c_lock);
     }
@@ -985,6 +983,17 @@ pmDiscoverNewSource(pmDiscover *p, int context)
     pmDiscoverInvokeSourceCallBacks(p, &timestamp);
 }
 
+static char *
+archive_dir_lock_path(pmDiscover *p)
+{
+    char	path[MAXNAMELEN], lockpath[MAXNAMELEN];
+    int		sep = pmPathSeparator();
+
+    strncpy(path, p->context.name, sizeof(path)-1);
+    pmsprintf(lockpath, sizeof(lockpath), "%s%c%s", dirname(path), sep, "lock");
+    return strndup(lockpath, sizeof(lockpath));
+}
+
 /*
  * Process metadata records until EOF. That can span multiple
  * callbacks if we get a partial record read.
@@ -1007,6 +1016,7 @@ process_metadata(pmDiscover *p)
     __pmLogHdr		hdr;
     sds			msg, source;
     static uint32_t	*buf = NULL;
+    char		*lock_path;
     int			deleted;
     struct stat		sbuf;
     static int		buflen = 0;
@@ -1020,7 +1030,12 @@ process_metadata(pmDiscover *p)
     if (pmDebugOptions.discovery)
 	fprintf(stderr, "process_metadata: %s in progress %s\n",
 		p->context.name, pmDiscoverFlagsStr(p));
+    pmDiscoverStatsAdd(p->module, "metadata.callbacks", NULL, 1);
+    lock_path = archive_dir_lock_path(p);
     for (;;) {
+	if (lock_path && access(lock_path, F_OK) == 0)
+	    break;
+	pmDiscoverStatsAdd(p->module, "metadata.loops", NULL, 1);
 	off = lseek(p->fd, 0, SEEK_CUR);
 	nb = read(p->fd, &hdr, sizeof(__pmLogHdr));
 
@@ -1036,6 +1051,7 @@ process_metadata(pmDiscover *p)
 	    /* rewind so we can wait for more data on the next change CallBack */
 	    lseek(p->fd, off, SEEK_SET);
 	    partial = 1;
+	    pmDiscoverStatsAdd(p->module, "metadata.partial_reads", NULL, 1);
 	    continue;
 	}
 
@@ -1045,6 +1061,7 @@ process_metadata(pmDiscover *p)
 	    /* rewind and wait for more data, as above */
 	    lseek(p->fd, off, SEEK_SET);
 	    partial = 1;
+	    pmDiscoverStatsAdd(p->module, "metadata.partial_reads", NULL, 1);
 	    continue;
 	}
 
@@ -1067,6 +1084,7 @@ process_metadata(pmDiscover *p)
 	    /* rewind and wait for more data, as above */
 	    lseek(p->fd, off, SEEK_SET);
 	    partial = 1;
+	    pmDiscoverStatsAdd(p->module, "metadata.partial_reads", NULL, 1);
 	    continue;
 	}
 
@@ -1078,6 +1096,7 @@ process_metadata(pmDiscover *p)
 	    /* decode pmDesc result from PDU buffer */
 	    nnames = 0;
 	    names = NULL;
+	    pmDiscoverStatsAdd(p->module, "metadata.decode.desc", NULL, 1);
 	    if ((e = pmDiscoverDecodeMetaDesc(buf, len, &desc, &nnames, &names)) < 0) {
 		if (pmDebugOptions.discovery)
 		    fprintf(stderr, "%s failed: err=%d %s\n",
@@ -1102,6 +1121,7 @@ process_metadata(pmDiscover *p)
 
 	case TYPE_INDOM:
 	    /* decode indom result from buffer */
+	    pmDiscoverStatsAdd(p->module, "metadata.decode.indom", NULL, 1);
 	    if ((e = pmDiscoverDecodeMetaInDom(buf, len, &ts, &inresult)) < 0) {
 		if (pmDebugOptions.discovery)
 		    fprintf(stderr, "%s failed: err=%d %s\n",
@@ -1113,6 +1133,7 @@ process_metadata(pmDiscover *p)
 
 	case TYPE_LABEL:
 	    /* decode labelset from buffer */
+	    pmDiscoverStatsAdd(p->module, "metadata.decode.label", NULL, 1);
 	    if ((e = pmDiscoverDecodeMetaLabelSet(buf, len, &ts, &id, &type, &nsets, &labelset)) < 0) {
 		if (pmDebugOptions.discovery)
 		    fprintf(stderr, "%s failed: err=%d %s\n",
@@ -1147,6 +1168,7 @@ process_metadata(pmDiscover *p)
 		fprintf(stderr, "TEXT\n");
 	    /* decode help text from buffer */
 	    buffer = NULL;
+	    pmDiscoverStatsAdd(p->module, "metadata.decode.helptext", NULL, 1);
 	    if ((e = pmDiscoverDecodeMetaHelpText(buf, len, &type, &id, &buffer)) < 0) {
 		if (pmDebugOptions.discovery)
 		    fprintf(stderr, "%s failed: err=%d %s\n",
@@ -1182,11 +1204,27 @@ process_metadata(pmDiscover *p)
 	/* flag that all available metadata has now been read */
 	p->flags &= ~PM_DISCOVER_FLAGS_META_IN_PROGRESS;
 
+    if (lock_path)
+    	free(lock_path);
+
     if (pmDebugOptions.discovery)
 	fprintf(stderr, "%s: completed, partial=%d %s %s\n",
 			"process_metadata", partial, p->context.name, pmDiscoverFlagsStr(p));
 }
 
+static void
+bump_logvol_decode_stats(pmDiscover *p, pmResult *r)
+{
+    if (r->numpmid == 0)
+	pmDiscoverStatsAdd(p->module, "logvol.decode.mark_record", NULL, 1);
+    else if (r->numpmid < 0)
+	pmDiscoverStatsAdd(p->module, "logvol.decode.error_result", NULL, 1);
+    else {
+	pmDiscoverStatsAdd(p->module, "logvol.decode.result", NULL, 1);
+	pmDiscoverStatsAdd(p->module, "logvol.decode.result_pmids", NULL, r->numpmid);
+    }
+}
+
 /*
  * Fetch metric values to EOF and call all registered callbacks.
  * Always process metadata thru to EOF before any logvol data.
@@ -1195,19 +1233,26 @@ static void
 process_logvol(pmDiscover *p)
 {
     int			sts;
-    pmResult		*r;
+    pmResult		*r = NULL;
     pmTimespec		ts;
     int			oldcurvol;
     __pmContext		*ctxp;
     __pmArchCtl		*acp;
+    char		*lock_path;
 
+    pmDiscoverStatsAdd(p->module, "logvol.callbacks", NULL, 1);
+    lock_path = archive_dir_lock_path(p);
     for (;;) {
+	if (lock_path && access(lock_path, F_OK) == 0)
+	    break;
+	pmDiscoverStatsAdd(p->module, "logvol.loops", NULL, 1);
 	pmUseContext(p->ctx);
 	ctxp = __pmHandleToPtr(p->ctx);
 	acp = ctxp->c_archctl;
 	oldcurvol = acp->ac_curvol;
 	PM_UNLOCK(ctxp->c_lock);
 
+	r = NULL; /* so we know if pmFetchArchive() assigned it */
 	if ((sts = pmFetchArchive(&r)) < 0) {
 	    /* err handling to skip to the next vol */
 	    ctxp = __pmHandleToPtr(p->ctx);
@@ -1215,6 +1260,7 @@ process_logvol(pmDiscover *p)
 	    if (oldcurvol < acp->ac_curvol) {
 	    	__pmLogChangeVol(acp, acp->ac_curvol);
 		acp->ac_offset = 0; /* __pmLogFetch will fix it up */
+		pmDiscoverStatsAdd(p->module, "logvol.change_vol", NULL, 1);
 	    }
 	    PM_UNLOCK(ctxp->c_lock);
 
@@ -1237,6 +1283,7 @@ process_logvol(pmDiscover *p)
 	    }
 
 	    /* we are done - return and wait for another callback */
+	    r = NULL;
 	    break;
 	}
 
@@ -1253,17 +1300,30 @@ process_logvol(pmDiscover *p)
 	}
 
 	/*
-	 * TODO: persistently save current timestamp, so after being restarted,
-	 * pmproxy can resume where it left off for each archive.
+	 * TODO (perhaps): persistently save current timestamp, so after being
+	 * restarted, pmproxy can resume where it left off for each archive.
 	 */
 	ts.tv_sec = r->timestamp.tv_sec;
 	ts.tv_nsec = r->timestamp.tv_usec * 1000;
+	bump_logvol_decode_stats(p, r);
+	pmDiscoverInvokeValuesCallBack(p, &ts, r);
+	pmFreeResult(r);
+	r = NULL;
+    }
+
+    if (r) {
+	ts.tv_sec = r->timestamp.tv_sec;
+	ts.tv_nsec = r->timestamp.tv_usec * 1000;
+	bump_logvol_decode_stats(p, r);
 	pmDiscoverInvokeValuesCallBack(p, &ts, r);
 	pmFreeResult(r);
     }
 
     /* datavol is now up-to-date and at EOF */
     p->flags &= ~PM_DISCOVER_FLAGS_DATAVOL_READY;
+
+    if (lock_path)
+    	free(lock_path);
 }
 
 static void
@@ -1273,6 +1333,10 @@ pmDiscoverInvokeCallBacks(pmDiscover *p)
     sds			msg;
     sds			metaname;
 
+    check_deleted(p);
+    if (p->flags & PM_DISCOVER_FLAGS_DELETED)
+    	return; /* ignore deleted archive */
+
     if (p->ctx < 0) {
 	/*
 	 * once off initialization on the first event
@@ -1282,24 +1346,27 @@ pmDiscoverInvokeCallBacks(pmDiscover *p)
 
 	    /* create the PMAPI context (once off) */
 	    if ((sts = pmNewContext(p->context.type, p->context.name)) < 0) {
-		/*
-		 * Likely an early callback on a new (still empty) archive.
-		 * If so, just ignore the callback and don't log any scary
-		 * looking messages. We'll get another CB soon.
-		 */
-		if (sts != PM_ERR_NODATA || pmDebugOptions.desperate) {
-		    infofmt(msg, "pmNewContext failed for %s: %s\n",
-			    p->context.name, pmErrStr(sts));
-		    moduleinfo(p->module, PMLOG_ERROR, msg, p->data);
+		if (sts == -ENOENT) {
+		    /* newly deleted archive */
+		    p->flags |= PM_DISCOVER_FLAGS_DELETED;
 		}
+		else {
+		    /*
+		     * Likely an early callback on a new (still empty) archive.
+		     * If so, just ignore the callback and don't log any scary
+		     * looking messages. We'll get another CB soon.
+		     */
+		    if (sts != PM_ERR_NODATA || pmDebugOptions.desperate) {
+			infofmt(msg, "pmNewContext failed for %s: %s\n",
+				p->context.name, pmErrStr(sts));
+			moduleinfo(p->module, PMLOG_ERROR, msg, p->data);
+		    }
+		}
+		/* no further processing for this archive */
 		return;
 	    }
-
-	    /*
-	     * We have a valid pmapi context. Initialize context state
-	     * and invoke registered source callbacks.
-	     */
-	    pmDiscoverNewSource(p, sts);
+	    pmDiscoverStatsAdd(p->module, "logvol.new_contexts", NULL, 1);
+	    p->ctx = sts;
 
 	    if ((sts = pmGetArchiveEnd(&tvp)) < 0) {
 		/* Less likely, but could still be too early (as above) */
@@ -1308,9 +1375,17 @@ pmDiscoverInvokeCallBacks(pmDiscover *p)
 		moduleinfo(p->module, PMLOG_ERROR, msg, p->data);
 		pmDestroyContext(p->ctx);
 		p->ctx = -1;
+		pmDiscoverStatsAdd(p->module, "logvol.get_archive_end_failed", NULL, 1);
+
 		return;
 	    }
 
+	    /*
+	     * We have a valid pmapi context. Initialize context state
+	     * and invoke registered source callbacks.
+	     */
+	    pmDiscoverNewSource(p, p->ctx);
+
 	    /* seek to end of archive for logvol data - see also TODO in process_logvol() */
 	    pmSetMode(PM_MODE_FORW, &tvp, 1);
 
@@ -1322,8 +1397,12 @@ pmDiscoverInvokeCallBacks(pmDiscover *p)
 	    metaname = sdsnew(p->context.name);
 	    metaname = sdscat(metaname, ".meta");
 	    if ((p->fd = open(metaname, O_RDONLY)) < 0) {
-		infofmt(msg, "open failed for %s: %s\n", metaname, osstrerror());
-		moduleinfo(p->module, PMLOG_ERROR, msg, p->data);
+		if (p->fd == -ENOENT)
+		    p->flags |= PM_DISCOVER_FLAGS_DELETED;
+		else {
+		    infofmt(msg, "open failed for %s: %s\n", metaname, osstrerror());
+		    moduleinfo(p->module, PMLOG_ERROR, msg, p->data);
+		}
 		sdsfree(metaname);
 		return;
 	    }
@@ -1413,6 +1492,24 @@ directory_changed_cb(pmDiscover *p, void *arg)
 static void
 changed_callback(pmDiscover *p)
 {
+    time_t		now;
+    int			throttle;
+    int			n_purged;
+
+    /* dynamic callback throttle - to improve scaling */
+    if ((throttle = n_monitored / 40) < 1)
+	throttle = 1;
+    pmDiscoverStatsSet(p->module, "throttle", NULL, throttle);
+    if ((p->flags & PM_DISCOVER_FLAGS_NEW) == 0) {
+	if (time(&now) - p->lastcb < throttle || 
+	    pmDiscoverGetInflightRedisRequests(p->module) > 1000000) {
+	    pmDiscoverStatsAdd(p->module, "throttled_changed_callbacks", NULL, 1);
+	    return; /* throttled */
+	}
+    }
+    p->lastcb = now;
+
+    pmDiscoverStatsAdd(p->module, "changed_callbacks", NULL, 1);
     if (pmDebugOptions.discovery)
 	fprintf(stderr, "CHANGED %s (%s)\n", p->context.name,
 			pmDiscoverFlagsStr(p));
@@ -1423,7 +1520,6 @@ changed_callback(pmDiscover *p)
 	 * in due course by pmDiscoverPurgeDeleted.
 	 */
 	return;
-	
     }
 
     if (p->flags & PM_DISCOVER_FLAGS_COMPRESSED) {
@@ -1440,6 +1536,7 @@ changed_callback(pmDiscover *p)
 	    fprintf(stderr, "%s DIRECTORY CHANGED %s (%s)\n",
 	    	stamp(), p->context.name, pmDiscoverFlagsStr(p));
 	}
+
 	pmDiscoverArchives(p->context.name, p->module, p->data);
 	pmDiscoverTraverse(PM_DISCOVER_FLAGS_NEW, created_callback);
 
@@ -1451,7 +1548,10 @@ changed_callback(pmDiscover *p)
 	    directory_changed_cb, (void *)p->context.name);
 
 	/* finally, purge deleted entries (globally), if any */
-	pmDiscoverPurgeDeleted();
+	n_purged = pmDiscoverPurgeDeleted();
+	pmDiscoverStatsAdd(p->module, "purged", NULL, n_purged);
+	n_monitored -= n_purged;
+	pmDiscoverStatsSet(p->module, "monitored", NULL, n_monitored);
     }
 
     if (pmDebugOptions.discovery) {
@@ -1465,6 +1565,7 @@ static void
 dir_callback(pmDiscover *p)
 {
     pmDiscoverMonitor(p->context.name, changed_callback);
+    p->flags &= ~PM_DISCOVER_FLAGS_NEW;
 }
 
 static void
@@ -1474,6 +1575,7 @@ archive_callback(pmDiscover *p)
 	if (pmDebugOptions.discovery)
 	    fprintf(stderr, "DISCOVERED ARCHIVE %s\n", p->context.name);
 	pmDiscoverMonitor(p->context.name, changed_callback);
+	p->flags &= ~PM_DISCOVER_FLAGS_NEW;
     }
 }
 
