@@ -18,9 +18,6 @@
 #include "libpcp.h"
 #include "mmv_stats.h"
 #include "slots.h"
-#ifdef HAVE_REGEX_H
-#include <regex.h>
-#endif
 #ifdef HAVE_LIBUV
 #include <uv.h>
 #else
@@ -63,8 +60,8 @@ typedef enum pmDiscoverFlags {
     PM_DISCOVER_FLAGS_DATAVOL			= (1 << 5), /* archive data volume */
     PM_DISCOVER_FLAGS_INDEX			= (1 << 6), /* archive index file */
     PM_DISCOVER_FLAGS_META			= (1 << 7), /* archive metadata */
-    PM_DISCOVER_FLAGS_DATAVOL_READY		= (1 << 9), /* flag: datavol data available */
-    PM_DISCOVER_FLAGS_META_IN_PROGRESS		= (1 << 8), /* flag: metadata read in progress */
+    PM_DISCOVER_FLAGS_DATAVOL_READY		= (1 << 8), /* flag: datavol data available */
+    PM_DISCOVER_FLAGS_META_IN_PROGRESS		= (1 << 9), /* flag: metadata read in progress */
 
     PM_DISCOVER_FLAGS_ALL			= ((unsigned int)~PM_DISCOVER_FLAGS_NONE)
 } pmDiscoverFlags;
@@ -87,6 +84,7 @@ typedef struct pmDiscover {
 #ifdef HAVE_LIBUV
     uv_fs_event_t		*event_handle;	/* uv fs_notify event handle */ 
 #endif
+    time_t			lastcb;		/* time last callback processed */
     struct stat			statbuf;	/* stat buffer */
     void			*baton;		/* private internal lib data */
     void			*data;		/* opaque user data pointer */
@@ -119,12 +117,13 @@ extern void pmSearchDiscoverText(pmDiscoverEvent *,
 typedef struct discoverModuleData {
     unsigned int		handle;		/* callbacks context handle */
     unsigned int		shareslots;	/* boolean, sharing 'slots' */
-    sds				logname;	/* archive directory dirname */
     mmv_registry_t		*metrics;	/* registry of metrics */
+    void			*metrics_handle;
     struct dict			*config;	/* configuration dict */
     uv_loop_t			*events;	/* event library loop */
     redisSlots			*slots;		/* server slots data */
-    regex_t			exclude_names;	/* metric names to exclude */
+    unsigned int		exclude_names;	/* exclude metric names */
+    sds				*patterns;	/* metric name patterns */
     struct dict			*pmids;		/* dict of excluded PMIDs */
     unsigned int		exclude_indoms;	/* exclude instance domains */
     struct dict			*indoms;	/* dict of excluded InDoms */
