@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2017-2018 Red Hat.
+ * Copyright (c) 2017-2020 Red Hat.
+ * Copyright (c) 2020 Yushan ZHANG.
  * 
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,6 +17,10 @@
 
 #include "pmapi.h"
 #include "pmwebapi.h"
+#include "batons.h"
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+#endif
 
 /*
  * Time series querying
@@ -55,6 +60,11 @@ typedef enum nodetype {
     N_RESCALE,
     N_SCALE,
     N_DEFINED,
+    N_ABS,
+    N_FLOOR,
+    N_LOG,
+    N_SQRT,
+    N_ROUND,
 
 /* node_t time-related sub-types */
     N_RANGE = 100,
@@ -76,6 +86,15 @@ typedef enum nodetype {
     MAX_NODETYPE
 } nodetype_t;
 
+typedef struct seriesGetSID {
+    seriesBatonMagic	header;		/* MAGIC_SID */
+    sds			name;		/* series or source SID */
+    sds			metric;		/* back-pointer for instance series */
+    /* various flags */
+    int			freed : 1;	/* freed individually on completion */
+    void		*baton;
+} seriesGetSID;
+
 typedef struct meta {
     int			type;	/* PM_TYPE_* */
     int			sem;	/* PM_SEM_* */
@@ -87,6 +106,45 @@ typedef struct series_set {
     int			nseries;
 } series_set_t;
 
+typedef struct series_instance_set {
+    /* Number of series instances */
+    int			num_instances;
+    pmSeriesValue	*series_instance;
+} series_instance_set_t;
+
+typedef struct series_sample_set {
+    seriesGetSID		*sid;
+    sds				metric_name;
+    pmSeriesDesc		series_desc;
+    void			*baton;
+    int				compatibility;
+    /* Number of series samples */
+    int				num_samples;
+    series_instance_set_t	*series_sample;
+} series_sample_set_t;
+
+typedef struct series_value_set {
+    /* Number of series identifiers*/
+    int				num_series;
+    series_sample_set_t		*series_values;
+} series_value_set_t;
+
+
+typedef struct timing {
+    /* input string */
+    pmSeriesTimeWindow	window;
+
+    /* parsed inputs */
+    struct timeval	delta;	
+    struct timeval	align;
+    struct timeval	start;
+    struct timeval	end;
+    unsigned int	count;		/* sample count */
+    unsigned int	offset;		/* sample offset */
+    int			zone;		/* pmNewZone handle */
+} timing_t;
+
+
 typedef struct node {
     enum nodetype	type;
     enum nodetype	subtype;
@@ -104,23 +162,16 @@ typedef struct node {
     /* partial match data for glob/regex */
     int			nmatches;
     sds			*matches;
-    void		*regex;	/* compiled regex */
+    regex_t		regex;	/* compiled regex */
     unsigned long long	cursor;
+
+    /* result set of time series values at this node */
+    series_value_set_t	value_set;
+
+    /* Corresponding time specifier */
+    timing_t		time;
 } node_t;
 
-typedef struct timing {
-    /* input string */
-    pmSeriesTimeWindow	window;
-
-    /* parsed inputs */
-    struct timeval	delta;	
-    struct timeval	align;
-    struct timeval	start;
-    struct timeval	end;
-    int			count;		/* sample count */
-    int			offset;		/* sample offset */
-    int			zone;		/* pmNewZone handle */
-} timing_t;
 
 typedef struct series {
     sds			name;
@@ -128,6 +179,7 @@ typedef struct series {
     timing_t		time;
 } series_t;
 
+extern int series_parse(sds, series_t *, char **, void *);
 extern int series_solve(pmSeriesSettings *, node_t *, timing_t *, pmSeriesFlags, void *);
 extern int series_load(pmSeriesSettings *, node_t *, timing_t *, pmSeriesFlags, void *);
 
