@@ -256,6 +256,8 @@ default_labelset(context_t *c, pmLabelSet **sets)
 	*sets = lp;
 	return 0;
     }
+    if (lp)
+	free(lp); /* Coverity CID340558 */
     return sts;
 }
 
@@ -270,15 +272,15 @@ metric_labelsets(metric_t *metric, char *buffer, int length,
     indom_t	*indom = metric->indom;
     int		nsets = 0;
 
-    if (context->labelset)
+    if (context && context->labelset)
 	sets[nsets++] = context->labelset;
-    if (domain->labelset)
+    if (domain && domain->labelset)
 	sets[nsets++] = domain->labelset;
     if (indom && indom->labelset)
 	sets[nsets++] = indom->labelset;
-    if (cluster->labelset)
+    if (cluster && cluster->labelset)
 	sets[nsets++] = cluster->labelset;
-    if (metric->labelset)
+    if (metric && metric->labelset)
 	sets[nsets++] = metric->labelset;
 
     return pmMergeLabelSets(sets, nsets, buffer, length, filter, arg);
@@ -1248,8 +1250,8 @@ pmwebapi_new_pmid(context_t *cp, const sds name, pmID pmid,
 {
     struct metric	*mp = NULL;
     pmDesc		desc;
-    char		**names, errmsg[PM_MAXERRMSGLEN], buffer[64];
-    int			sts, numnames;
+    char		**names = NULL, errmsg[PM_MAXERRMSGLEN], buffer[64];
+    int			sts, numnames = 0;
 
     if ((sts = pmUseContext(cp->context)) < 0) {
 	fprintf(stderr, "%s: failed to use context for PMID %s: %s\n",
@@ -1274,7 +1276,8 @@ pmwebapi_new_pmid(context_t *cp, const sds name, pmID pmid,
 		    pmErrStr_r(sts, errmsg, sizeof(errmsg)));
     } else {
 	mp = pmwebapi_new_metric(cp, name, &desc, numnames, names);
-	free(names);
+	if (numnames > 0 && names != NULL)
+	    free(names);
     }
     return mp;
 }
