@@ -46,18 +46,27 @@ enum profile { PROFILE_ADD, PROFILE_DEL };
 typedef struct webgroups {
     struct dict		*contexts;
     mmv_registry_t	*metrics;
+    void		*metrics_handle;
     struct dict		*config;
     uv_loop_t		*events;
     unsigned int	active;
     uv_timer_t		timer;
+    uv_mutex_t		mutex;
+    int			stats_timer;
 } webgroups;
 
 static struct webgroups *
 webgroups_lookup(pmWebGroupModule *module)
 {
-    if (module->privdata == NULL)
+    struct webgroups *groups = module->privdata;
+
+    if (module->privdata == NULL) {
 	module->privdata = calloc(1, sizeof(struct webgroups));
-    return (struct webgroups *)module->privdata;
+	groups = (struct webgroups *)module->privdata;
+	uv_mutex_init(&groups->mutex);
+	groups->stats_timer = -1;
+    }
+    return groups;
 }
 
 static int
@@ -93,8 +102,11 @@ webgroup_drop_context(struct context *context, struct webgroups *groups)
 	    context->garbage = 1;
 	    uv_timer_stop(&context->timer);
 	}
-	if (groups)
-	    dictUnlink(groups->contexts, &context->randomid);
+	if (groups) {
+	    uv_mutex_lock(&groups->mutex);
+	    dictDelete(groups->contexts, &context->randomid);
+	    uv_mutex_unlock(&groups->mutex);
+	}
 	uv_close((uv_handle_t *)&context->timer, webgroup_release_context);
     }
 }
@@ -126,7 +138,7 @@ webgroup_access(struct context *cp, sds hostspec, dict *params,
 {
     __pmHashNode	*node;
     __pmHashCtl		attrs;
-    pmHostSpec		*hosts = NULL;
+    __pmHostSpec	*hosts = NULL;
     char		*msg, buf[512];
     int			sts, bytes, numhosts = 0;
     sds			value;
@@ -206,13 +218,16 @@ webgroup_new_context(pmWebGroupSettings *sp, dict *params,
     cp->context = -1;
     cp->timeout = polltime;
 
+    uv_mutex_lock(&groups->mutex);
     if ((cp->randomid = random()) < 0 ||
 	dictFind(groups->contexts, &cp->randomid) != NULL) {
 	infofmt(*message, "random number failure on new web context");
 	pmwebapi_free_context(cp);
 	*status = -ESRCH;
+	uv_mutex_unlock(&groups->mutex);
 	return NULL;
     }
+    uv_mutex_unlock(&groups->mutex);
     cp->origin = sdscatfmt(sdsempty(), "%i", cp->randomid);
     cp->name.sds = sdsdup(hostspec ? hostspec : LOCALHOST);
     cp->realm = sdscatfmt(sdsempty(), "pmapi/%i", cp->randomid);
@@ -241,7 +256,9 @@ webgroup_new_context(pmWebGroupSettings *sp, dict *params,
 	pmwebapi_free_context(cp);
 	return NULL;
     }
+    uv_mutex_lock(&groups->mutex);
     dictAdd(groups->contexts, &cp->randomid, cp);
+    uv_mutex_unlock(&groups->mutex);
 
     /* leave until the end because uv_timer_init makes this visible in uv_run */
     handle = (uv_handle_t *)&cp->timer;
@@ -260,27 +277,52 @@ webgroup_new_context(pmWebGroupSettings *sp, dict *params,
 static void
 webgroup_garbage_collect(struct webgroups *groups)
 {
-    dictIterator        *iterator = dictGetSafeIterator(groups->contexts);
+    dictIterator        *iterator;
     dictEntry           *entry;
     context_t		*cp;
 
     if (pmDebugOptions.http || pmDebugOptions.libweb)
 	fprintf(stderr, "%s: started\n", "webgroup_garbage_collect");
 
-    while ((entry = dictNext(iterator)) != NULL) {
-	cp = (context_t *)dictGetVal(entry);
-	if (cp->garbage && cp->privdata == groups) {
-	    if (pmDebugOptions.http || pmDebugOptions.libweb)
-		fprintf(stderr, "GC context %u (%p)\n", cp->randomid, cp);
-	    webgroup_drop_context(cp, groups);
+    /* do context GC if we get the lock (else don't block here) */
+    if (uv_mutex_trylock(&groups->mutex) == 0) {
+	iterator = dictGetSafeIterator(groups->contexts);
+	for (entry = dictNext(iterator); entry;) {
+	    cp = (context_t *)dictGetVal(entry);
+	    entry = dictNext(iterator);
+	    if (cp->garbage && cp->privdata == groups) {
+		if (pmDebugOptions.http || pmDebugOptions.libweb)
+		    fprintf(stderr, "GC context %u (%p)\n", cp->randomid, cp);
+		uv_mutex_unlock(&groups->mutex);
+		webgroup_drop_context(cp, groups);
+		uv_mutex_lock(&groups->mutex);
+	    }
 	}
+	dictReleaseIterator(iterator);
+	uv_mutex_unlock(&groups->mutex);
     }
-    dictReleaseIterator(iterator);
 
     if (pmDebugOptions.http || pmDebugOptions.libweb)
 	fprintf(stderr, "%s: finished\n", "webgroup_garbage_collect");
 }
 
+static void
+refresh_maps_metrics(void *data)
+{
+    struct webgroups	*groups = (struct webgroups *)data;
+
+    if (groups->metrics_handle) {
+	mmv_stats_set(groups->metrics_handle, "contextmap.size",
+	    NULL, dictSize(contextmap));
+	mmv_stats_set(groups->metrics_handle, "namesmap.size",
+	    NULL, dictSize(namesmap));
+	mmv_stats_set(groups->metrics_handle, "labelsmap.size",
+	    NULL, dictSize(labelsmap));
+	mmv_stats_set(groups->metrics_handle, "instmap.size",
+	    NULL, dictSize(instmap));
+    }
+}
+
 static void
 webgroup_worker(uv_timer_t *arg)
 {
@@ -349,6 +391,8 @@ webgroup_lookup_context(pmWebGroupSettings *sp, sds *id, dict *params,
 	groups->timer.data = (void *)groups;
 	uv_timer_start(&groups->timer, webgroup_worker,
 			default_worker, default_worker);
+	/* timer for map stats refresh */
+	groups->stats_timer = pmWebTimerRegister(refresh_maps_metrics, (void *)groups);
     }
 
     if (*id == NULL) {
@@ -1936,12 +1980,16 @@ struct instore {
     int			status;
 };
 
+#define STORE_INST_MAX	100000	/* cap number of storable instances */
+
 static int
 store_add_instid(struct instore *store, int id)
 {
     int		*insts, count = store->count++;
 
-    if ((insts = realloc(store->insts, store->count * sizeof(int))) == NULL) {
+    if ((store->count >= STORE_INST_MAX) ||
+	(insts = realloc(store->insts, store->count * sizeof(int))) == NULL) {
+	store->status = -E2BIG;
 	store->count = count;	/* reset */
 	return count;
     }
@@ -2262,6 +2310,43 @@ pmWebGroupSetConfiguration(pmWebGroupModule *module, dict *config)
     return -ENOMEM;
 }
 
+static void
+pmWebGroupSetupMetrics(pmWebGroupModule *module)
+{
+    struct webgroups	*webgroups = webgroups_lookup(module);
+    pmUnits            nounits = MMV_UNITS(0,0,0,0,0,0);
+    pmInDom            noindom = MMV_INDOM_NULL;
+
+    if (webgroups == NULL || webgroups->metrics == NULL)
+	return; /* no metric registry has been set up */
+
+    /*
+     * Reverse mapping dict metrics
+     */
+    mmv_stats_add_metric(webgroups->metrics, "contextmap.size", 1,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+	"context map dictionary size",
+	"number of entries in the context map dictionary");
+
+    mmv_stats_add_metric(webgroups->metrics, "namesmap.size", 2,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+	"metric names map dictionary size",
+	"number of entries in the metric names map dictionary");
+
+    mmv_stats_add_metric(webgroups->metrics, "labelsmap.size", 3,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+	"labels map dictionary size",
+	"number of entries in the labels map dictionary");
+
+    mmv_stats_add_metric(webgroups->metrics, "instmap.size", 4,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+	"instance name map dictionary size",
+	"number of entries in the instance name map dictionary");
+
+    webgroups->metrics_handle = mmv_stats_start(webgroups->metrics);
+}
+
+
 int
 pmWebGroupSetMetricRegistry(pmWebGroupModule *module, mmv_registry_t *registry)
 {
@@ -2269,6 +2354,7 @@ pmWebGroupSetMetricRegistry(pmWebGroupModule *module, mmv_registry_t *registry)
 
     if (webgroups) {
 	webgroups->metrics = registry;
+	pmWebGroupSetupMetrics(module);
 	return 0;
     }
     return -ENOMEM;
@@ -2286,6 +2372,8 @@ pmWebGroupClose(pmWebGroupModule *module)
 	if (groups->active) {
 	    groups->active = 0;
 	    uv_timer_stop(&groups->timer);
+	    pmWebTimerRelease(groups->stats_timer);
+	    groups->stats_timer = -1;
 	}
 	iterator = dictGetIterator(groups->contexts);
 	while ((entry = dictNext(iterator)) != NULL)
