@@ -42,6 +42,13 @@ pmdaIndom indomtable[] = {
 	{ .it_indom = SBD_DEVICE_INDOM },
 	{ .it_indom = DRBD_RESOURCE_INDOM },
 	{ .it_indom = DRBD_PEER_DEVICE_INDOM },
+	{ .it_indom = PACEMAKER_CONSTRAINTS_ALL_INDOM },
+	{ .it_indom = PACEMAKER_NODE_ATTRIB_ALL_INDOM },
+	{ .it_indom = PACEMAKER_RESOURCES_ALL_INDOM },
+	{ .it_indom = COROSYNC_RING_ALL_INDOM},
+	{ .it_indom = SBD_DEVICE_ALL_INDOM},
+	{ .it_indom = DRBD_RESOURCE_ALL_INDOM},
+	{ .it_indom = DRBD_PEER_DEVICE_ALL_INDOM},
 };
 
 #define INDOM(x) (indomtable[x].it_indom)
@@ -83,6 +90,10 @@ pmdaMetric metrictable[] = {
 		PMDA_PMID(CLUSTER_PACEMAKER_CONSTRAINTS, PACEMAKER_CONSTRAINTS_SCORE),
 		PM_TYPE_STRING, PACEMAKER_CONSTRAINTS_INDOM, PM_SEM_INSTANT,
 		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
+	{ .m_desc = {
+		PMDA_PMID(CLUSTER_PACEMAKER_CONSTRAINTS_ALL, 0),
+		PM_TYPE_U32, PACEMAKER_CONSTRAINTS_ALL_INDOM, PM_SEM_INSTANT,
+		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
 	{ .m_desc = {
 		PMDA_PMID(CLUSTER_PACEMAKER_NODES, PACEMAKER_NODES_ONLINE),
 		PM_TYPE_U32, PACEMAKER_NODES_INDOM, PM_SEM_INSTANT,
@@ -127,6 +138,10 @@ pmdaMetric metrictable[] = {
 		PMDA_PMID(CLUSTER_PACEMAKER_NODE_ATTRIB, PACEMAKER_NODES_ATTRIB_VALUE),
 		PM_TYPE_STRING, PACEMAKER_NODE_ATTRIB_INDOM, PM_SEM_INSTANT,
 		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
+	{ .m_desc = {
+		PMDA_PMID(CLUSTER_PACEMAKER_NODE_ATTRIB_ALL, 0),
+		PM_TYPE_U32, PACEMAKER_NODE_ATTRIB_ALL_INDOM, PM_SEM_INSTANT,
+		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },	
 	{ .m_desc = {
 		PMDA_PMID(CLUSTER_PACEMAKER_RESOURCES, PACEMAKER_RESOURCES_AGENT),
 		PM_TYPE_STRING, PACEMAKER_RESOURCES_INDOM, PM_SEM_INSTANT,
@@ -139,6 +154,10 @@ pmdaMetric metrictable[] = {
 		PMDA_PMID(CLUSTER_PACEMAKER_RESOURCES, PACEMAKER_RESOURCES_GROUP),
 		PM_TYPE_STRING, PACEMAKER_RESOURCES_INDOM, PM_SEM_INSTANT,
 		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
+	{ .m_desc = {
+		PMDA_PMID(CLUSTER_PACEMAKER_RESOURCES_ALL, 0),
+		PM_TYPE_U32, PACEMAKER_RESOURCES_ALL_INDOM, PM_SEM_INSTANT,
+		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
 	{ .m_desc = {
 		PMDA_PMID(CLUSTER_PACEMAKER_RESOURCES, PACEMAKER_RESOURCES_MANAGED),
 		PM_TYPE_U32, PACEMAKER_RESOURCES_INDOM, PM_SEM_INSTANT,
@@ -228,6 +247,10 @@ pmdaMetric metrictable[] = {
 		PMDA_PMID(CLUSTER_COROSYNC_RING, COROSYNC_RINGS_RING_ID),
 		PM_TYPE_STRING, COROSYNC_RING_INDOM, PM_SEM_INSTANT,
 		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
+	{ .m_desc = {
+		PMDA_PMID(CLUSTER_COROSYNC_RING_ALL, 0),
+		PM_TYPE_U32, COROSYNC_RING_ALL_INDOM, PM_SEM_INSTANT,
+		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },	
 	/* SBD */
 	{ .m_desc = {
 		PMDA_PMID(CLUSTER_SBD_DEVICE, SBD_DEVICE_PATH),
@@ -237,6 +260,10 @@ pmdaMetric metrictable[] = {
 		PMDA_PMID(CLUSTER_SBD_DEVICE, SBD_DEVICE_STATUS),
 		PM_TYPE_STRING, SBD_DEVICE_INDOM, PM_SEM_INSTANT,
 		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
+	{ .m_desc = {
+		PMDA_PMID(CLUSTER_SBD_DEVICE_ALL, 0),
+		PM_TYPE_U32, SBD_DEVICE_ALL_INDOM, PM_SEM_INSTANT,
+		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
 	{ .m_desc = {
 		PMDA_PMID(CLUSTER_SBD_DEVICE, SBD_DEVICE_TIMEOUT_MSGWAIT),
 		PM_TYPE_U32, SBD_DEVICE_INDOM, PM_SEM_INSTANT,
@@ -270,6 +297,10 @@ pmdaMetric metrictable[] = {
 		PMDA_PMID(CLUSTER_DRBD_RESOURCE, DRBD_RESOURCE_DISK_STATE),
 		PM_TYPE_STRING, DRBD_RESOURCE_INDOM, PM_SEM_INSTANT,
 		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
+	{ .m_desc ={
+		PMDA_PMID(CLUSTER_DRBD_RESOURCE_ALL, 0),
+		PM_TYPE_U32, DRBD_RESOURCE_ALL_INDOM, PM_SEM_INSTANT,
+		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
 	{ .m_desc = {
 		PMDA_PMID(CLUSTER_DRBD_RESOURCE, DRBD_RESOURCE_WRITTEN),
 		PM_TYPE_U32, DRBD_RESOURCE_INDOM, PM_SEM_INSTANT,
@@ -318,6 +349,10 @@ pmdaMetric metrictable[] = {
 		PMDA_PMID(CLUSTER_DRBD_PEER_DEVICE, DRBD_PEER_DEVICE_PEER_DISK_STATE),
 		PM_TYPE_STRING, DRBD_PEER_DEVICE_INDOM, PM_SEM_INSTANT,
 		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
+	{ .m_desc = {
+		PMDA_PMID(CLUSTER_DRBD_PEER_DEVICE_ALL, 0),
+		PM_TYPE_U32, DRBD_PEER_DEVICE_ALL_INDOM, PM_SEM_INSTANT,
+		PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
 	{ .m_desc = {
 		PMDA_PMID(CLUSTER_DRBD_PEER_DEVICE, DRBD_PEER_DEVICE_CONNECTIONS_SYNC),
 		PM_TYPE_FLOAT, DRBD_PEER_DEVICE_INDOM, PM_SEM_INSTANT,
@@ -359,10 +394,16 @@ hacluster_pacemaker_fail_instance_refresh(void)
 	FILE		*pf;
 	pmInDom		indom = INDOM(PACEMAKER_FAIL_INDOM);
 
-	pmsprintf(buffer, sizeof(buffer), "%s", crm_mon_command);
+	/*
+	 * Update indom cache based off the reading of crm_mon listed in
+	 * the output from crm_mon
+	 */
+	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", crm_mon_command);
 
 	if ((pf = popen(buffer, "r")) == NULL)
-		return -oserror();
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* First we need to check whether we are in <node_history> section*/
@@ -425,12 +466,20 @@ hacluster_pacemaker_constraints_instance_refresh(void)
 	int			found_constraints = 0;
 	FILE		*pf;
 	pmInDom		indom = INDOM(PACEMAKER_CONSTRAINTS_INDOM);
+	pmInDom		indom_all = INDOM(PACEMAKER_CONSTRAINTS_ALL_INDOM);
 
-	pmsprintf(buffer, sizeof(buffer), "%s", cibadmin_command);
+	/*
+	 * Update indom cache based off the reading of cibadmin listed in
+	 * the output from cibadmin
+	 */
+	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	pmdaCacheOp(indom_all, PMDA_CACHE_INACTIVE);
+
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", cibadmin_command);
 	buffer[sizeof(buffer)-1] = '\0';
 
 	if ((pf = popen(buffer, "r")) == NULL)
-		return -oserror();
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* First we need to check whether we are in <constraints> section*/
@@ -457,6 +506,7 @@ hacluster_pacemaker_constraints_instance_refresh(void)
 				continue;
 
 			pmdaCacheStore(indom, PMDA_CACHE_ADD, constraint_name, (void *)constraints);
+			pmdaCacheStore(indom_all, PMDA_CACHE_ADD, constraint_name, NULL);
 		}
 	}
 	pclose(pf);
@@ -472,10 +522,16 @@ hacluster_pacemaker_nodes_instance_refresh(void)
 	FILE		*pf;
 	pmInDom		indom = INDOM(PACEMAKER_NODES_INDOM);
 
-	pmsprintf(buffer, sizeof(buffer), "%s", crm_mon_command);
+	/*
+	 * Update indom cache based off the reading of crm_mon listed in
+	 * the output from crm_mon
+	 */
+	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", crm_mon_command);
 
 	if ((pf = popen(buffer, "r")) == NULL)
-		return -oserror();
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* First we need to check whether we are in <nodes> section*/
@@ -524,11 +580,19 @@ hacluster_pacemaker_node_attrib_instance_refresh(void)
 	int			found_node_attributes = 0, found_node_name = 0;
 	FILE		*pf;
 	pmInDom		indom = INDOM(PACEMAKER_NODE_ATTRIB_INDOM);
+	pmInDom		indom_all = INDOM(PACEMAKER_NODE_ATTRIB_ALL_INDOM);
 
-	pmsprintf(buffer, sizeof(buffer), "%s", crm_mon_command);
+	/*
+	 * Update indom cache based off the reading of crm_mon listed in
+	 * the output from crm_mon
+	 */
+	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	pmdaCacheOp(indom_all, PMDA_CACHE_INACTIVE);
+
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", crm_mon_command);
 
 	if ((pf = popen(buffer, "r")) == NULL)
-		return -oserror();
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* First we need to check whether we are in <node_history> section*/
@@ -582,6 +646,7 @@ hacluster_pacemaker_node_attrib_instance_refresh(void)
 					continue;
 
 				pmdaCacheStore(indom, PMDA_CACHE_ADD, instance_name, (void *)node_attrib);
+				pmdaCacheStore(indom_all, PMDA_CACHE_ADD, instance_name, NULL);
 			}
 		}
 	}
@@ -597,11 +662,19 @@ hacluster_pacemaker_resources_instance_refresh(void)
 	int			found_resources = 0;
 	FILE		*pf;
 	pmInDom		indom= INDOM(PACEMAKER_RESOURCES_INDOM);
+	pmInDom		indom_all = INDOM(PACEMAKER_RESOURCES_ALL_INDOM);
 
-	pmsprintf(buffer, sizeof(buffer), "%s", crm_mon_command);
+	/*
+	 * Update indom cache based off the reading of crm_mon listed in
+	 * the output from crm_mon
+	 */
+	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	pmdaCacheOp(indom_all, PMDA_CACHE_INACTIVE);
+
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", crm_mon_command);
 
 	if ((pf = popen(buffer, "r")) == NULL)
-		return -oserror();
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* First we need to check whether we are in <resources> section*/
@@ -652,6 +725,7 @@ hacluster_pacemaker_resources_instance_refresh(void)
 					continue;
 
 				pmdaCacheStore(indom, PMDA_CACHE_ADD, instance_name, (void *)pace_resources);
+				pmdaCacheStore(indom_all, PMDA_CACHE_ADD, instance_name, NULL);
 
 				/* Clear node name in the event that a resource has not got a node attachment */
 				memset(node_name, '\0', sizeof(node_name));
@@ -676,9 +750,11 @@ hacluster_corosync_node_instance_refresh(void)
 	 * membership information section of corosync-quorumtool output
 	 */
 	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", quorumtool_command);
 
-	if ((pf = popen(quorumtool_command, "r")) == NULL)
-		return -oserror();
+	if ((pf = popen(buffer, "r")) == NULL)
+		return oserror();
 
 	while (fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* Clear whitespace at start of each line */
@@ -728,15 +804,19 @@ hacluster_corosync_ring_instance_refresh(void)
 	char		buffer[4096], ring_name[128];
 	FILE		*pf;
 	pmInDom		indom = INDOM(COROSYNC_RING_INDOM);
+	pmInDom		indom_all = INDOM(COROSYNC_RING_ALL_INDOM);
 
 	/*
 	 * Update indom cache based off number of nodes listed in the
 	 * membership information section of corosync-quorumtool output
 	 */
 	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	pmdaCacheOp(indom_all, PMDA_CACHE_INACTIVE);
 	
-	if ((pf = popen(cfgtool_command, "r")) == NULL)
-		return -oserror();
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", cfgtool_command);
+	
+	if ((pf = popen(buffer, "r")) == NULL)
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 
@@ -770,6 +850,7 @@ hacluster_corosync_ring_instance_refresh(void)
 				continue;
 
 			pmdaCacheStore(indom, PMDA_CACHE_ADD, ring_name, (void *)ring);
+			pmdaCacheStore(indom_all, PMDA_CACHE_ADD, ring_name, NULL);
 		}
 	}
 	pclose(pf);
@@ -785,12 +866,14 @@ hacluster_sbd_device_instance_refresh(void)
 	char		*buffer_ptr;
 	FILE		*fp;
 	pmInDom		indom = INDOM(SBD_DEVICE_INDOM);
+	pmInDom		indom_all = INDOM(SBD_DEVICE_ALL_INDOM);
 
 	/*
 	 * Update indom cache based off number of nodes listed in the
 	 * membership information section of corosync-quorumtool output
 	 */
 	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	pmdaCacheOp(indom_all, PMDA_CACHE_INACTIVE);
 
 	if ((fp = fopen(sbd_path, "r")) == NULL)
 		/*
@@ -838,6 +921,7 @@ hacluster_sbd_device_instance_refresh(void)
 						continue;
 
 					pmdaCacheStore(indom, PMDA_CACHE_ADD, dev_name, (void *)sbd);
+					pmdaCacheStore(indom_all, PMDA_CACHE_ADD, dev_name, NULL);
 				}
 			}
 		}
@@ -856,6 +940,7 @@ hacluster_drbd_resource_instance_refresh(void)
 	char		*buffer_ptr;
 	FILE		*pf;
 	pmInDom		indom = INDOM(DRBD_RESOURCE_INDOM);
+	pmInDom		indom_all = INDOM(DRBD_RESOURCE_ALL_INDOM);
 
 	int found_node = 0, found_volume = 0, nesting = 0;
 
@@ -864,9 +949,12 @@ hacluster_drbd_resource_instance_refresh(void)
 	 * the json output from drbdsetup
 	 */
 	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	pmdaCacheOp(indom_all, PMDA_CACHE_INACTIVE);
 	
-	if ((pf = popen(drbdsetup_command, "r")) == NULL)
-		return -oserror();
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", drbdsetup_command);
+	
+	if ((pf = popen(buffer, "r")) == NULL)
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* Clear whitespace at start of each line */
@@ -915,6 +1003,8 @@ hacluster_drbd_resource_instance_refresh(void)
 				continue;
 
 			pmdaCacheStore(indom, PMDA_CACHE_ADD, resource_name, (void *)resource);
+			pmdaCacheStore(indom_all, PMDA_CACHE_ADD, resource_name, NULL);
+
 			found_volume = 0;
 		}
 	}
@@ -931,6 +1021,7 @@ hacluster_drbd_peer_device_instance_refresh(void)
 	char		*buffer_ptr;
 	FILE		*pf;
 	pmInDom		indom = INDOM(DRBD_PEER_DEVICE_INDOM);
+	pmInDom		indom_all = INDOM(DRBD_PEER_DEVICE_ALL_INDOM);
 
 	int found_node = 0, found_peer_node = 0, nesting = 0;
 	
@@ -939,9 +1030,12 @@ hacluster_drbd_peer_device_instance_refresh(void)
 	 * the json output from drbdsetup
 	 */
 	pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+	pmdaCacheOp(indom_all, PMDA_CACHE_INACTIVE);
 
-	if ((pf = popen(drbdsetup_command, "r")) == NULL)
-		return -oserror();
+	pmsprintf(buffer, sizeof(buffer), "%s 2>&1", drbdsetup_command);
+
+	if ((pf = popen(buffer, "r")) == NULL)
+		return oserror();
 
 	while(fgets(buffer, sizeof(buffer)-1, pf) != NULL) {
 		/* Clear whitespace at start of each line */
@@ -990,6 +1084,8 @@ hacluster_drbd_peer_device_instance_refresh(void)
 				continue;
 
 			pmdaCacheStore(indom, PMDA_CACHE_ADD, peer_name, (void *)peer_device);
+			pmdaCacheStore(indom_all, PMDA_CACHE_ADD, peer_name, NULL);
+
 			found_peer_node = 0;
 		}
 	}
@@ -1080,7 +1176,8 @@ hacluster_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 		if (!pmdaCacheLookup(INDOM(PACEMAKER_CONSTRAINTS_INDOM), i, &constraint_name, (void **)&constraints) || !constraints)
 			continue;
 
-		if (need_refresh[CLUSTER_PACEMAKER_CONSTRAINTS])
+		if (need_refresh[CLUSTER_PACEMAKER_CONSTRAINTS] ||
+			need_refresh[CLUSTER_PACEMAKER_CONSTRAINTS_ALL])
 			hacluster_refresh_pacemaker_constraints(constraint_name, &constraints->location_constraints);
 	}
 	
@@ -1100,7 +1197,8 @@ hacluster_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 		if (!pmdaCacheLookup(INDOM(PACEMAKER_NODE_ATTRIB_INDOM), i, &attrib_name, (void **)&node_attribs) || !node_attribs)
 			continue;
 
-		if (need_refresh[CLUSTER_PACEMAKER_NODE_ATTRIB])
+		if (need_refresh[CLUSTER_PACEMAKER_NODE_ATTRIB] ||
+			need_refresh[CLUSTER_PACEMAKER_NODE_ATTRIB_ALL])
 			hacluster_refresh_pacemaker_node_attribs(attrib_name, &node_attribs->attributes);
 	}
 	
@@ -1110,7 +1208,8 @@ hacluster_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 		if (!pmdaCacheLookup(INDOM(PACEMAKER_RESOURCES_INDOM), i, &pace_resource_name, (void **)&pace_resources) || !pace_resources)
 			continue;
 
-		if (need_refresh[CLUSTER_PACEMAKER_RESOURCES])
+		if (need_refresh[CLUSTER_PACEMAKER_RESOURCES] ||
+			need_refresh[CLUSTER_PACEMAKER_RESOURCES_ALL])
 			hacluster_refresh_pacemaker_resources(pace_resource_name, &pace_resources->resources);
 	}
 
@@ -1133,7 +1232,8 @@ hacluster_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 		if (!pmdaCacheLookup(INDOM(COROSYNC_RING_INDOM), i, &ring_name, (void **)&ring) || !ring)
 			continue;
 
-		if (need_refresh[CLUSTER_COROSYNC_RING])
+		if (need_refresh[CLUSTER_COROSYNC_RING] ||
+			need_refresh[CLUSTER_COROSYNC_RING_ALL])
 			hacluster_refresh_corosync_ring(ring_name, &ring->rings);
 	}
 	
@@ -1143,7 +1243,8 @@ hacluster_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 		if (!pmdaCacheLookup(INDOM(SBD_DEVICE_INDOM), i, &sbd_dev, (void **)&sbd) || !sbd)
 			continue;
 
-		if (need_refresh[CLUSTER_SBD_DEVICE])
+		if (need_refresh[CLUSTER_SBD_DEVICE] ||
+			need_refresh[CLUSTER_SBD_DEVICE_ALL])
 			hacluster_refresh_sbd_device(sbd_dev, &sbd->sbd);
 	}
 	
@@ -1153,7 +1254,8 @@ hacluster_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 		if (!pmdaCacheLookup(INDOM(DRBD_RESOURCE_INDOM), i, &resource_name, (void **)&resource) || !resource)
 			continue;
 
-		if (need_refresh[CLUSTER_DRBD_RESOURCE])
+		if (need_refresh[CLUSTER_DRBD_RESOURCE] ||
+			need_refresh[CLUSTER_DRBD_RESOURCE_ALL])
 			hacluster_refresh_drbd_resource(resource_name, &resource->resource);
 	}
 	
@@ -1163,7 +1265,8 @@ hacluster_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 		if (!pmdaCacheLookup(INDOM(DRBD_PEER_DEVICE_INDOM), i, &peer_device, (void **)&peer) || !peer)
 			continue;
 
-		if (need_refresh[CLUSTER_DRBD_PEER_DEVICE])
+		if (need_refresh[CLUSTER_DRBD_PEER_DEVICE] ||
+			need_refresh[CLUSTER_DRBD_PEER_DEVICE_ALL])
 			hacluster_refresh_drbd_peer_device(peer_device, &peer->peer_device);
 	}
 	
@@ -1220,6 +1323,9 @@ hacluster_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 				return sts;
 			return hacluster_pacemaker_constraints_fetch(item, &constraints->location_constraints, atom);
 			
+		case CLUSTER_PACEMAKER_CONSTRAINTS_ALL:
+			return hacluster_pacemaker_constraints_all_fetch(item, atom);			
+			
 		case CLUSTER_PACEMAKER_NODES:
 			sts = pmdaCacheLookup(INDOM(PACEMAKER_NODES_INDOM), inst, NULL, (void **)&pace_nodes);
 			if (sts < 0)
@@ -1231,6 +1337,9 @@ hacluster_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 			if (sts < 0)
 				return sts;
 			return hacluster_pacemaker_node_attribs_fetch(item, &pace_attribs->attributes, atom);
+
+		case CLUSTER_PACEMAKER_NODE_ATTRIB_ALL:
+			return hacluster_pacemaker_node_attribs_all_fetch(item, atom);
 			
 		case CLUSTER_PACEMAKER_RESOURCES:
 			sts = pmdaCacheLookup(INDOM(PACEMAKER_RESOURCES_INDOM), inst, NULL, (void **)&pace_resources);
@@ -1238,6 +1347,9 @@ hacluster_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 				return sts;
 			return hacluster_pacemaker_resources_fetch(item, &pace_resources->resources, atom);
 
+		case CLUSTER_PACEMAKER_RESOURCES_ALL:
+			return hacluster_pacemaker_resources_all_fetch(item, atom);
+
 		case CLUSTER_COROSYNC_NODE:
 			sts = pmdaCacheLookup(INDOM(COROSYNC_NODE_INDOM), inst, NULL, (void **)&node);
 			if (sts < 0)
@@ -1252,18 +1364,27 @@ hacluster_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 			if (sts < 0)
 				return sts;
 			return hacluster_corosync_ring_fetch(item, &ring->rings, atom);
+
+		case CLUSTER_COROSYNC_RING_ALL:
+			return hacluster_corosync_ring_all_fetch(item, atom);
 			
 		case CLUSTER_SBD_DEVICE:
 			sts = pmdaCacheLookup(INDOM(SBD_DEVICE_INDOM), inst, NULL, (void **)&sbd);
 			if (sts < 0)
 				return sts;
 			return hacluster_sbd_device_fetch(item, &sbd->sbd, atom);
+
+		case CLUSTER_SBD_DEVICE_ALL:
+			return hacluster_sbd_device_all_fetch(item, atom);
 			
 		case CLUSTER_DRBD_RESOURCE:
 			sts = pmdaCacheLookup(INDOM(DRBD_RESOURCE_INDOM), inst, NULL, (void **)&resource);
 			if (sts < 0)
 				return sts;
-			return hacluster_drbd_resource_fetch(item, &resource->resource, atom);		
+			return hacluster_drbd_resource_fetch(item, &resource->resource, atom);	
+
+		case CLUSTER_DRBD_RESOURCE_ALL:
+			return hacluster_drbd_resource_all_fetch(item, atom);	
 		
 		case CLUSTER_DRBD_PEER_DEVICE:
 			sts = pmdaCacheLookup(INDOM(DRBD_PEER_DEVICE_INDOM), inst, NULL, (void **)&peer);
@@ -1271,6 +1392,9 @@ hacluster_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 				return sts;
 			return hacluster_drbd_peer_device_fetch(item, &peer->peer_device, atom);
 
+		case CLUSTER_DRBD_PEER_DEVICE_ALL:
+			return hacluster_drbd_peer_device_all_fetch(item, atom);
+
 		default:
 			return PM_ERR_PMID;
 	}
@@ -1278,6 +1402,220 @@ hacluster_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	return PMDA_FETCH_STATIC;
 }
 
+static int
+hacluster_labelInDom(pmID pmid, pmLabelSet **lp)
+{	
+	unsigned int cluster = pmID_cluster(pmid);
+
+	switch (cluster) {
+		case CLUSTER_PACEMAKER_CONSTRAINTS_ALL:
+			pmdaAddLabels(lp, "{\"constraint\":\"constraint\"}");
+			pmdaAddLabels(lp, "{\"node\":\"node name\"}");
+			pmdaAddLabels(lp, "{\"resource\":\"resource name\"}");
+			pmdaAddLabels(lp, "{\"role\":\"role of node\"}");
+			pmdaAddLabels(lp, "{\"score\":\"score\"}");
+			return 1;
+
+		case CLUSTER_PACEMAKER_NODE_ATTRIB_ALL:
+			pmdaAddLabels(lp, "{\"name\":\"attribute name\"}");
+			pmdaAddLabels(lp, "{\"node\":\"node name\"}");
+			pmdaAddLabels(lp, "{\"value\":\"value\"}");
+			return 1;
+
+		case CLUSTER_PACEMAKER_RESOURCES_ALL:
+			pmdaAddLabels(lp, "{\"agent\":\"agent\"}");
+			pmdaAddLabels(lp, "{\"clone\":\"clone\"}");
+			pmdaAddLabels(lp, "{\"group\":\"group\"}");
+			pmdaAddLabels(lp, "{\"managed\":\"managed\"}");
+			pmdaAddLabels(lp, "{\"node\":\"node name\"}");
+			pmdaAddLabels(lp, "{\"resource\":\"resource name\"}");
+			pmdaAddLabels(lp, "{\"role\":\"role\"}");
+			return 1;
+
+		case CLUSTER_COROSYNC_RING_ALL:
+			pmdaAddLabels(lp, "{\"address\":\"ip address\"}");
+			pmdaAddLabels(lp, "{\"node_id\":\"id of node\"}");
+			pmdaAddLabels(lp, "{\"number\":\"ring number\"}");
+			pmdaAddLabels(lp, "{\"ring_id\":\"id of ring\"}");
+			return 1;
+
+		case CLUSTER_SBD_DEVICE_ALL:
+			pmdaAddLabels(lp, "{\"device\":\"device\"}");
+			pmdaAddLabels(lp, "{\"status\":\"status\"}");
+			return 1;
+
+		case CLUSTER_DRBD_RESOURCE_ALL:
+			pmdaAddLabels(lp, "{\"disk_state\":\"disk state\"}");
+			pmdaAddLabels(lp, "{\"resource\":\"resource name\"}");
+			pmdaAddLabels(lp, "{\"role\":\"role\"}");
+			pmdaAddLabels(lp, "{\"volume\":\"volume\"}");
+			return 1;
+
+		case CLUSTER_DRBD_PEER_DEVICE_ALL:
+			pmdaAddLabels(lp, "{\"peer_disk_state\":\"peer disk state\"}");
+			pmdaAddLabels(lp, "{\"peer_node_id\":\"peer node id\"}");
+			pmdaAddLabels(lp, "{\"peer_role\":\"peer role\"}");
+			pmdaAddLabels(lp, "{\"resource\":\"resource\"}");
+			pmdaAddLabels(lp, "{\"volume\":\"volume\"}");
+			return 1;
+
+		default:
+			break;
+	}
+	return 0;
+}
+			
+static int
+hacluster_label(int ident, int type, pmLabelSet **lpp, pmdaExt *pmda)
+{
+	int sts;
+
+	switch (type) {
+		case PM_LABEL_ITEM:
+			if ((sts = hacluster_labelInDom((pmID)ident, lpp)) <0)
+				return sts;
+			break;
+
+		default:
+			break;
+	}
+	return pmdaLabel(ident, type, lpp, pmda);
+}
+
+static int
+hacluster_labelCallBack(pmInDom indom, unsigned int inst, pmLabelSet **lp)
+{
+	struct location_constraints		*constraints;
+	struct attributes 				*attributes;
+	struct resources 				*resources;
+	struct rings					*ring;
+	struct sbd						*sbd;
+	struct resource					*resource;
+	struct peer_device				*peer_device;
+
+	int 							sts;
+	int								no_node_attachment = 0;
+	char 							*name;
+	char 							*node_name, *attribute_name, *node, *resource_id, *tofree, *str;
+
+	if (indom == PM_INDOM_NULL)
+		return 0;
+
+	switch (pmInDom_serial(indom)) {
+		case PACEMAKER_CONSTRAINTS_ALL_INDOM:
+			sts = pmdaCacheLookup(INDOM(PACEMAKER_CONSTRAINTS_INDOM), inst, &name, (void **)&constraints);
+			if (sts < 0 || sts == PMDA_CACHE_INACTIVE)
+				return 0;
+			return pmdaAddLabels(lp, "{\"constraint\":\"%s\", \"node\":\"%s\", \"resource\":\"%s\", \"role\":\"%s\", \"score\":\"%s\"}",
+				name,
+				constraints->node,
+				constraints->resource,
+				constraints->role,
+				constraints->score
+			);
+
+		case PACEMAKER_NODE_ATTRIB_ALL_INDOM:
+			sts = pmdaCacheLookup(INDOM(PACEMAKER_NODE_ATTRIB_INDOM), inst, &name, (void **)&attributes);
+			if (sts < 0 || sts == PMDA_CACHE_INACTIVE)
+				return 0;
+			/* 
+	 		 * We need to split our combined NODE:ATTRIBUTE_NAME instance names into their
+	 		 * separated NODE and ATTRIBUTE_NAME fields
+	 		 */
+			tofree = str = strdup(name);
+			attribute_name = strsep(&str, ":");
+			node_name = strsep(&str, ":");
+
+			sts = pmdaAddLabels(lp, "{\"name\":\"%s\", \"node\":\"%s\", \"value\":\"%s\"}",
+				node_name,
+				attribute_name,
+				attributes->value
+			);
+			free(tofree);
+			return sts;
+			
+		case PACEMAKER_RESOURCES_ALL_INDOM:
+			sts = pmdaCacheLookup(INDOM(PACEMAKER_RESOURCES_INDOM), inst, &name, (void **)&resources);
+			if (sts < 0 || sts == PMDA_CACHE_INACTIVE)
+				return 0;
+				
+			if (strchr(name, ':') == NULL) {
+				node = (char*)name;
+				no_node_attachment = 1;
+				resource_id = "";
+			} else {
+				tofree = str = strdup(name);
+				node = strsep(&str, ":");
+				resource_id = strsep(&str, ":");
+			}
+				
+			sts = pmdaAddLabels(lp, "{\"agent\":\"%s\", \"clone\":\"%s\", \"group\":\"%s\", \"managed\":%u, \"node\":\"%s\", \"resource\":\"%s\", \"role\":\"%s\"}",
+				resources->agent,
+				resources->clone,
+				resources->group,
+				resources->managed,
+				resource_id,
+				node, 
+				resources->role
+			);
+			
+			if (!no_node_attachment)
+				free(tofree);
+			return sts;
+			
+		case COROSYNC_RING_ALL_INDOM:
+			sts = pmdaCacheLookup(INDOM(COROSYNC_RING_INDOM), inst, &name, (void **)&ring);
+			if (sts <0 || sts == PMDA_CACHE_INACTIVE)
+				return 0;
+				
+			return pmdaAddLabels(lp, "{\"address\":\"%s\", \"node_id\":%"PRIu64", \"number\":%u, \"ring_id\":\"%s\"}",
+				ring->address,
+				ring->node_id,
+				ring->number,
+				ring->ring_id
+			);
+
+		case SBD_DEVICE_ALL_INDOM:
+			sts = pmdaCacheLookup(INDOM(SBD_DEVICE_INDOM), inst, &name, (void**)&sbd);
+			if (sts <0 || sts == PMDA_CACHE_INACTIVE)
+				return 0;
+				
+			return pmdaAddLabels(lp, "{\"device\":\"%s\", \"status\":\"%s\"}",
+				sbd->path,
+				sbd->status
+			);
+
+		case DRBD_RESOURCE_ALL_INDOM:
+			sts = pmdaCacheLookup(INDOM(DRBD_RESOURCE_INDOM), inst, &name, (void**)&resource);
+			if (sts <0 || sts == PMDA_CACHE_INACTIVE)
+				return 0;
+				
+			return pmdaAddLabels(lp, "{\"disk_state\":\"%s\", \"resource\":\"%s\", \"role\":\"%s\", \"volume\":\"%s\"}",
+				resource->disk_state,
+				resource->resource,
+				resource->role,
+				resource->volume
+			);
+			
+		case DRBD_PEER_DEVICE_ALL_INDOM:
+			sts = pmdaCacheLookup(INDOM(DRBD_PEER_DEVICE_INDOM), inst, &name, (void**)&peer_device);
+			if (sts <0 || sts == PMDA_CACHE_INACTIVE)
+				return 0;
+				
+			return pmdaAddLabels(lp, "{\"peer_disk_state\":\"%s\", \"peer_node_id\":\"%s\", \"peer_role\":\"%s\", \"resource\":\"%s\", \"volume\":%"PRIu32"}",
+				peer_device->peer_disk_state,
+				peer_device->peer_node_id,
+				peer_device->peer_role,
+				peer_device->resource,
+				peer_device->volume
+			);
+
+		default:
+			break;
+	}
+	return 0;
+}
+
 void
 hacluster_inst_setup(void)
 {
@@ -1370,7 +1708,7 @@ hacluster_init(pmdaInterface *dp)
 		int sep = pmPathSeparator();
 		pmsprintf(helppath, sizeof(helppath), "%s%c" "hacluster" "%c" "help",
 			pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
-		pmdaDSO(dp, PMDA_INTERFACE_4, "HACLUSTER DSO", helppath);
+		pmdaDSO(dp, PMDA_INTERFACE_7, "HACLUSTER DSO", helppath);
 	}
 
 	if (dp->status != 0)
@@ -1383,13 +1721,15 @@ hacluster_init(pmdaInterface *dp)
 	sbd_stats_setup();
 	drbd_stats_setup();
 
-	dp->version.four.instance = hacluster_instance;
-	dp->version.four.fetch = hacluster_fetch;
-	dp->version.four.text = hacluster_text;
-	dp->version.four.pmid = hacluster_pmid;
-	dp->version.four.name = hacluster_name;
-	dp->version.four.children = hacluster_children;
+	dp->version.seven.instance = hacluster_instance;
+	dp->version.seven.fetch = hacluster_fetch;
+	dp->version.seven.text = hacluster_text;
+	dp->version.seven.pmid = hacluster_pmid;
+	dp->version.seven.name = hacluster_name;
+	dp->version.seven.children = hacluster_children;
+	dp->version.seven.label = hacluster_label;
 	pmdaSetFetchCallBack(dp, hacluster_fetchCallBack);
+	pmdaSetLabelCallBack(dp, hacluster_labelCallBack);
 
 	pmdaSetFlags(dp, PMDA_EXT_FLAG_HASHED);
 	pmdaInit(dp, indomtable, nindoms, metrictable, nmetrics);
@@ -1420,7 +1760,7 @@ main(int argc, char **argv)
 	pmSetProgname(argv[0]);
 	pmsprintf(helppath, sizeof(helppath), "%s%c" "hacluster" "%c" "help",
 		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
-	pmdaDaemon(&dispatch, PMDA_INTERFACE_4, pmGetProgname(), HACLUSTER, "hacluster.log", helppath);
+	pmdaDaemon(&dispatch, PMDA_INTERFACE_7, pmGetProgname(), HACLUSTER, "hacluster.log", helppath);
 
 	pmdaGetOptions(argc, argv, &opts, &dispatch);
 	if (opts.errors) {
