@@ -72,10 +72,13 @@ help_text_t  help_text[] = {
 { .name = "psinfo.tty",            .shorthelp = "controlling tty device number (zero if none)",        .longhelp = "" },
 { .name = "psinfo.cmin_flt",            .shorthelp = "count of minor page faults (i.e. reclaims) of all exited children",        .longhelp = "" },
 { .name = "psinfo.stime",            .shorthelp = "time (in ms) spent executing system code (calls) since process started",        .longhelp = "" },
-{ .name = "psinfo.policy",            .shorthelp = "Scheduling policy",        .longhelp = "" },
+{ .name = "psinfo.policy",            .shorthelp = "scheduling policy (/from /proc/<pid>/stat)",        .longhelp = "" },
+{ .name = "psinfo.policy_s",            .shorthelp = "scheduling policy in string form",        .longhelp = "" },
 { .name = "psinfo.cguest_time",            .shorthelp = "Guest time of the process’s children",        .longhelp = "" },
 { .name = "psinfo.environ",           .shorthelp = "process environment (from /proc/<pid>/environ ascii space replaces null).",        .longhelp = "" },
 { .name = "psinfo.labels",            .shorthelp = "list of processes security labels (from /proc/<pid>/attr/current)",        .longhelp = "" },
+{ .name = "psinfo.cwd",            .shorthelp = "current working directory (from /proc/<pid>/cwd)",        .longhelp = "" },
+{ .name = "psinfo.exe",            .shorthelp = "current process executable (from /proc/<pid>/exe)",        .longhelp = "" },
 { .name = "psinfo.oom_score",         .shorthelp = "out-of-memory process selection score (from /proc/<pid>/oom_score)",        .longhelp = "" },
 { .name = "psinfo.blocked",            .shorthelp = "the value in the blocked field of struct task_struct for the process",        .longhelp = "" },
 { .name = "cgroup.subsys.hierarchy",            .shorthelp = "subsystem hierarchy from /proc/cgroups",        .longhelp = "" },
@@ -128,4 +131,24 @@ help_text_t  help_text[] = {
 { .name = "namespaces.pgid",   .shorthelp = "descendant namespace process group ID hierarchy (/proc/<pid>/status)", .longhelp = "" },
 { .name = "namespaces.sid",   .shorthelp = "descendant namespace session ID hierarchy (/proc/<pid>/status)", .longhelp = "" },
 { .name = "namespaces.envid", .shorthelp = "OpenVZ container identifier", .longhelp = "" },
+{ .name = "smaps.rss",	.shorthelp = "amount of mapping that is currently resident (/proc/<pid>/smaps_rollup)",        .longhelp = "" },
+{ .name = "smaps.pss",	.shorthelp = "processes proportional share of mappings (/proc/<pid>/smaps_rollup)",        .longhelp = "The proportional set size (PSS) of a process is the count of pages it has\nin memory, where each page is divided by the number of processes sharing it.\nSo if a process has 1000 pages all to itself, and 1000 shared with one other\nprocess, its PSS will be 1500.\nNote that even a page which is part of a MAP_SHARED mapping, but has only\na single pte mapped, i.e.  is currently used by only one process, is accounted\nas private and not as shared.\n" },
+{ .name = "smaps.pss_anon",   .shorthelp = "Pss_Anon mappings size from /proc/<pid>/smaps_rollup", .longhelp = "" },
+{ .name = "smaps.pss_file",   .shorthelp = "Pss_File mappings size from /proc/<pid>/smaps_rollup", .longhelp = "" },
+{ .name = "smaps.pss_shmem",   .shorthelp = "Pss_Shmem mappings size from /proc/<pid>/smaps_rollup", .longhelp = "" },
+{ .name = "smaps.shared_clean",   .shorthelp = "Shared_Clean mappings size from /proc/<pid>/smaps_rollup", .longhelp = "" },
+{ .name = "smaps.shared_dirty",   .shorthelp = "Shared_Dirty mappings size from /proc/<pid>/smaps_rollup", .longhelp = "" },
+{ .name = "smaps.private_clean",   .shorthelp = "Private_Clean mappings size from /proc/<pid>/smaps_rollup", .longhelp = "" },
+{ .name = "smaps.private_dirty",   .shorthelp = "Private_Dirty mappings size from /proc/<pid>/smaps_rollup", .longhelp = "" },
+{ .name = "smaps.referenced",   .shorthelp = "Referenced mappings size from /proc/<pid>/smaps_rollup", .longhelp = "Referenced indicates the amount of memory currently marked as referenced\nor accessed." },
+{ .name = "smaps.anonymous",   .shorthelp = "Anonymous mappings size from /proc/<pid>/smaps_rollup", .longhelp = "Anonymous shows the amount of memory that does not belong to any file.\nEven a mapping associated with a file may contain anonymous pages: when MAP_PRIVATE\nand a page is modified, the file page is replaced by a private anonymous copy." },
+{ .name = "smaps.lazyfree",   .shorthelp = "LazyFree mappings size from /proc/<pid>/smaps_rollup", .longhelp = "LazyFree shows the amount of memory which is marked by madvise(MADV_FREE).\nThe memory isn't freed immediately with madvise(), rather it's freed in memory\npressure if the memory is clean.\n" },
+{ .name = "smaps.anonhugepages",   .shorthelp = "AnonHugePages mappings size from /proc/<pid>/smaps_rollup", .longhelp = "AnonHugePages shows the amount of memory backed by transparent hugepages." },
+{ .name = "smaps.shmempmdmapped",    .shorthelp = "ShmemPmdMapped mappings size from /proc/<pid>/smaps_rollup", .longhelp = "ShmemPmdMapped shows the amount of shared (shmem/tmpfs) memory backed by\nhuge pages.\n" },
+{ .name = "smaps.filepmdmapped",    .shorthelp = "FilePmdMapped mappings size from /proc/<pid>/smaps_rollup", .longhelp = "FilePmdMapped shows the amount of memory backed by filesystem pages." },
+{ .name = "smaps.shared_hugetlb",    .shorthelp = "Shared_Hugetlb mappings size from /proc/<pid>/smaps_rollup", .longhelp = "Shared_Hugetlb shows the amount of memory backed by shared hugetlbfs pages\nwhich is *not* counted in RSS or PSS fields for historical reasons.\n" },
+{ .name = "smaps.private_hugetlb",    .shorthelp = "Private_Hugetlb mappings size from /proc/<pid>/smaps_rollup", .longhelp = "Private_Hugetlb shows the amount of memory backed by private hugetlbfs pages\nwhich is *not* counted in RSS or PSS fields for historical reasons.\n" },
+{ .name = "smaps.swap",    .shorthelp = "Swap size from /proc/<pid>/smaps_rollup", .longhelp = "Swap shows how much would-be-anonymous memory is also used, but out on swap.\nFor shmem mappings, Swap includes also the size of the mapped (and not\nreplaced by copy-on-write) part of the underlying shmem object out on swap.\n" },
+{ .name = "smaps.swappss",    .shorthelp = "SwapPss size from /proc/<pid>/smaps_rollup", .longhelp = "SwapPss shows proportional swap share of this mapping. Unlike Swap, this\ndoes not take into account swapped out page of underlying shmem objects." },
+{ .name = "smaps.locked",    .shorthelp = "Locked mappings size from /proc/<pid>/smaps_rollup",   .longhelp = "Locked indicates whether the mapping is locked in memory or not." },
 };
