@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015 Red Hat.
+ * Copyright (c) 2015,2021 Red Hat.
  * Copyright (c) 2014-2015 Martins Innus.  All Rights Reserved.
  * Copyright (c) 2000,2004 Silicon Graphics, Inc.  All Rights Reserved.
  * 
@@ -38,6 +38,7 @@ enum {
     DYNPROC_GROUP_IO,
     DYNPROC_GROUP_SCHEDSTAT,
     DYNPROC_GROUP_NAMESPACE,
+    DYNPROC_GROUP_SMAPS,
 
     NUM_DYNPROC_GROUPS
 };
@@ -50,7 +51,7 @@ enum {
 };
 
 /*
- * Map proc cluster id's to new hotproc varients that don't conflict
+ * Map proc cluster ID's to new hotproc variants that don't conflict
  */
 
 static int proc_hotproc_cluster_list[][2] = {
@@ -63,6 +64,9 @@ static int proc_hotproc_cluster_list[][2] = {
 	{ CLUSTER_PID_IO,	    CLUSTER_HOTPROC_PID_IO },
 	{ CLUSTER_PID_FD,	    CLUSTER_HOTPROC_PID_FD },
 	{ CLUSTER_PID_OOM_SCORE,    CLUSTER_HOTPROC_PID_OOM_SCORE },
+	{ CLUSTER_PID_SMAPS,	    CLUSTER_HOTPROC_PID_SMAPS },
+	{ CLUSTER_PID_EXE,	    CLUSTER_HOTPROC_PID_EXE },
+	{ CLUSTER_PID_CWD,	    CLUSTER_HOTPROC_PID_CWD },
 };
 
 
@@ -130,19 +134,22 @@ static dynproc_metric_t psinfo_metrics[] = {
 	{ .name = "guest_time",	    .cluster = CLUSTER_PID_STAT,	.item=45 },
 	{ .name = "cguest_time",    .cluster = CLUSTER_PID_STAT,	.item=46 },
 	{ .name = "environ",        .cluster = CLUSTER_PID_STAT,	.item=47 },
+	{ .name = "policy_s",	    .cluster = CLUSTER_PID_STAT,	.item=48 },
 	{ .name = "signal_s",	    .cluster = CLUSTER_PID_STATUS,	.item=16 },
 	{ .name = "blocked_s",	    .cluster = CLUSTER_PID_STATUS,	.item=17 },
 	{ .name = "sigignore_s",    .cluster = CLUSTER_PID_STATUS,	.item=18 },
 	{ .name = "sigcatch_s",	    .cluster = CLUSTER_PID_STATUS,	.item=19 },
 	{ .name = "threads",	    .cluster = CLUSTER_PID_STATUS,	.item=28 },
-	{ .name = "cgroups",	    .cluster = CLUSTER_PID_CGROUP,	.item = PROC_PID_CGROUP },
-	{ .name = "labels",	    .cluster = CLUSTER_PID_LABEL,	.item = PROC_PID_LABEL },
+	{ .name = "cgroups",	    .cluster = CLUSTER_PID_CGROUP,	.item=0 },
+	{ .name = "labels",	    .cluster = CLUSTER_PID_LABEL,	.item=0 },
 	{ .name = "vctxsw",	    .cluster = CLUSTER_PID_STATUS,	.item=29 },
 	{ .name = "nvctxsw",	    .cluster = CLUSTER_PID_STATUS,	.item=30 },
-	{ .name = "cpusallowed",    .cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_CPUSALLOWED },
-	{ .name = "ngid",	    .cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NGID },
-        { .name = "tgid",	    .cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_TGID },
-	{ .name = "oom_score",	    .cluster = CLUSTER_PID_OOM_SCORE,	.item = PROC_PID_OOM_SCORE },
+	{ .name = "cpusallowed",    .cluster = CLUSTER_PID_STATUS,	.item=31 },
+	{ .name = "ngid",	    .cluster = CLUSTER_PID_STATUS,	.item=32 },
+        { .name = "tgid",	    .cluster = CLUSTER_PID_STATUS,	.item=41 },
+	{ .name = "oom_score",	    .cluster = CLUSTER_PID_OOM_SCORE,	.item=0 },
+	{ .name = "exe",	    .cluster = CLUSTER_PID_EXE,		.item=0 },
+	{ .name = "cwd",	    .cluster = CLUSTER_PID_CWD,		.item=0 },
 };
 
 static dynproc_metric_t id_metrics[] = {
@@ -166,36 +173,36 @@ static dynproc_metric_t id_metrics[] = {
 };
 
 static dynproc_metric_t memory_metrics[] = {
-        { .name = "size",   .cluster = CLUSTER_PID_STATM,  .item=0 },
-        { .name = "rss",    .cluster = CLUSTER_PID_STATM,  .item=1 },
-        { .name = "share",  .cluster = CLUSTER_PID_STATM,  .item=2 },
-        { .name = "textrss",.cluster = CLUSTER_PID_STATM,  .item=3 },
-        { .name = "librss", .cluster = CLUSTER_PID_STATM,  .item=4 },
-        { .name = "datrss", .cluster = CLUSTER_PID_STATM,  .item=5 },
-        { .name = "dirty",  .cluster = CLUSTER_PID_STATM,  .item=6 },
-        { .name = "maps",   .cluster = CLUSTER_PID_STATM,  .item=7 },
-        { .name = "vmsize", .cluster = CLUSTER_PID_STATUS,  .item=20 },
-        { .name = "vmlock", .cluster = CLUSTER_PID_STATUS,  .item=21 },
-        { .name = "vmrss",  .cluster = CLUSTER_PID_STATUS,  .item=22 },
-        { .name = "vmdata", .cluster = CLUSTER_PID_STATUS,  .item=23 },
-        { .name = "vmstack",.cluster = CLUSTER_PID_STATUS,  .item=24 },
-        { .name = "vmexe",  .cluster = CLUSTER_PID_STATUS,  .item=25 },
-        { .name = "vmlib",  .cluster = CLUSTER_PID_STATUS,  .item=26 },
-        { .name = "vmswap", .cluster = CLUSTER_PID_STATUS,  .item=27 },
-        { .name = "vmpeak",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMPEAK },
-	{ .name = "vmpin",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMPIN },
-	{ .name = "vmhwm",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMHWM },
-	{ .name = "vmpte",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMPTE },
-	{ .name = "vmreal",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMREAL },
-	{ .name = "vmnonlib",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMNONLIB },
+        { .name = "size",     .cluster = CLUSTER_PID_STATM,  .item=0 },
+        { .name = "rss",      .cluster = CLUSTER_PID_STATM,  .item=1 },
+        { .name = "share",    .cluster = CLUSTER_PID_STATM,  .item=2 },
+        { .name = "textrss",  .cluster = CLUSTER_PID_STATM,  .item=3 },
+        { .name = "librss",   .cluster = CLUSTER_PID_STATM,  .item=4 },
+        { .name = "datrss",   .cluster = CLUSTER_PID_STATM,  .item=5 },
+        { .name = "dirty",    .cluster = CLUSTER_PID_STATM,  .item=6 },
+        { .name = "maps",     .cluster = CLUSTER_PID_STATM,  .item=7 },
+        { .name = "vmsize",   .cluster = CLUSTER_PID_STATUS, .item=20 },
+        { .name = "vmlock",   .cluster = CLUSTER_PID_STATUS, .item=21 },
+        { .name = "vmrss",    .cluster = CLUSTER_PID_STATUS, .item=22 },
+        { .name = "vmdata",   .cluster = CLUSTER_PID_STATUS, .item=23 },
+        { .name = "vmstack",  .cluster = CLUSTER_PID_STATUS, .item=24 },
+        { .name = "vmexe",    .cluster = CLUSTER_PID_STATUS, .item=25 },
+        { .name = "vmlib",    .cluster = CLUSTER_PID_STATUS, .item=26 },
+        { .name = "vmswap",   .cluster = CLUSTER_PID_STATUS, .item=27 },
+        { .name = "vmpeak",   .cluster = CLUSTER_PID_STATUS, .item=33 },
+	{ .name = "vmpin",    .cluster = CLUSTER_PID_STATUS, .item=34 },
+	{ .name = "vmhwm",    .cluster = CLUSTER_PID_STATUS, .item=35 },
+	{ .name = "vmpte",    .cluster = CLUSTER_PID_STATUS, .item=36 },
+	{ .name = "vmreal",   .cluster = CLUSTER_PID_STATUS, .item=43 },
+	{ .name = "vmnonlib", .cluster = CLUSTER_PID_STATUS, .item=44 },
 };
 
 static dynproc_metric_t namespace_metrics[] = {
-        { .name = "tgid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSTGID },
-        { .name = "pid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSPID },
-        { .name = "pgid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSPGID },
-        { .name = "sid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSSID },
-        { .name = "envid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_ENVID },
+        { .name = "tgid",   .cluster = CLUSTER_PID_STATUS, .item=37 },
+        { .name = "pid",    .cluster = CLUSTER_PID_STATUS, .item=38 },
+        { .name = "pgid",   .cluster = CLUSTER_PID_STATUS, .item=39 },
+        { .name = "sid",    .cluster = CLUSTER_PID_STATUS, .item=40 },
+        { .name = "envid",  .cluster = CLUSTER_PID_STATUS, .item=42 },
 };
 
 static dynproc_metric_t io_metrics[] = {
@@ -218,6 +225,29 @@ static dynproc_metric_t schedstat_metrics[] = {
 	{ .name = "pcount",	.cluster = CLUSTER_PID_SCHEDSTAT,	.item=2 },
 };
 
+static dynproc_metric_t smaps_metrics[] = {
+	{ .name = "rss",             .cluster = CLUSTER_PID_SMAPS,  .item=0 },
+	{ .name = "pss",             .cluster = CLUSTER_PID_SMAPS,  .item=1 },
+	{ .name = "pss_anon",        .cluster = CLUSTER_PID_SMAPS,  .item=2 },
+	{ .name = "pss_file",        .cluster = CLUSTER_PID_SMAPS,  .item=3 },
+	{ .name = "pss_shmem",       .cluster = CLUSTER_PID_SMAPS,  .item=4 },
+	{ .name = "shared_clean",    .cluster = CLUSTER_PID_SMAPS,  .item=5 },
+	{ .name = "shared_dirty",    .cluster = CLUSTER_PID_SMAPS,  .item=6 },
+	{ .name = "private_clean",   .cluster = CLUSTER_PID_SMAPS,  .item=7 },
+	{ .name = "private_dirty",   .cluster = CLUSTER_PID_SMAPS,  .item=8 },
+	{ .name = "referenced",      .cluster = CLUSTER_PID_SMAPS,  .item=9 },
+	{ .name = "anonymous",       .cluster = CLUSTER_PID_SMAPS,  .item=10 },
+	{ .name = "lazyfree",        .cluster = CLUSTER_PID_SMAPS,  .item=11 },
+	{ .name = "anonhugepages",   .cluster = CLUSTER_PID_SMAPS,  .item=12 },
+	{ .name = "shmempmdmapped",  .cluster = CLUSTER_PID_SMAPS,  .item=13 },
+	{ .name = "filepmdmapped",   .cluster = CLUSTER_PID_SMAPS,  .item=14 },
+	{ .name = "shared_hugetlb",  .cluster = CLUSTER_PID_SMAPS,  .item=15 },
+	{ .name = "private_hugetlb", .cluster = CLUSTER_PID_SMAPS,  .item=16 },
+	{ .name = "swap",            .cluster = CLUSTER_PID_SMAPS,  .item=17 },
+	{ .name = "swappss",         .cluster = CLUSTER_PID_SMAPS,  .item=18 },
+	{ .name = "locked",          .cluster = CLUSTER_PID_SMAPS,  .item=19 },
+};
+
 static dynproc_group_t dynproc_groups[] = {
 	[DYNPROC_GROUP_PSINFO]    = { .name = "psinfo",	    .metrics = psinfo_metrics,	    .nmetrics = sizeof(psinfo_metrics)/sizeof(dynproc_metric_t)},
 	[DYNPROC_GROUP_ID]	  = { .name = "id",	    .metrics = id_metrics,	    .nmetrics = sizeof(id_metrics)/sizeof(dynproc_metric_t)},
@@ -226,6 +256,7 @@ static dynproc_group_t dynproc_groups[] = {
 	[DYNPROC_GROUP_FD]	  = { .name = "fd",	    .metrics = fd_metrics,	    .nmetrics = sizeof(fd_metrics)/sizeof(dynproc_metric_t)},
 	[DYNPROC_GROUP_SCHEDSTAT] = { .name = "schedstat",  .metrics = schedstat_metrics,   .nmetrics = sizeof(schedstat_metrics)/sizeof(dynproc_metric_t) },
 	[DYNPROC_GROUP_NAMESPACE] = { .name = "namespaces", .metrics = namespace_metrics,   .nmetrics = sizeof(namespace_metrics)/sizeof(dynproc_metric_t) },
+	[DYNPROC_GROUP_SMAPS]     = { .name = "smaps",	    .metrics = smaps_metrics,	    .nmetrics = sizeof(smaps_metrics)/sizeof(dynproc_metric_t)},
 };
 
 /*
