@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2013-2020 Red Hat.
+ * Copyright (c) 2013-2021 Red Hat.
  * Copyright (c) 1995-2004 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -87,6 +87,8 @@ static pmDesc	desctab[] = {
     { PMDA_PMID(0,24), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) },
 /* labels */
     { PMDA_PMID(0,25), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_INSTANT, PMDA_PMUNITS(0,0,0,0,0,0) },
+/* zoneinfo -- local timezone tzfile identification  -- for pmlogger timezone */
+    { PMDA_PMID(0,26), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) },
 
 /* pdu_in.error */
     { PMDA_PMID(1,0), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
@@ -128,6 +130,10 @@ static pmDesc	desctab[] = {
     { PMDA_PMID(1,19), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 /* pdu_in.label */
     { PMDA_PMID(1,20), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_in.highres_fetch */
+    { PMDA_PMID(1,21), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_in.highres_result */
+    { PMDA_PMID(1,22), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 
 /* pdu_out.error */
     { PMDA_PMID(2,0), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
@@ -169,6 +175,10 @@ static pmDesc	desctab[] = {
     { PMDA_PMID(2,19), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 /* pdu_out.label */
     { PMDA_PMID(2,20), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_out.highres_fetch */
+    { PMDA_PMID(2,21), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_out.highres_result */
+    { PMDA_PMID(2,22), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 
 /* pmlogger.port */
     { PMDA_PMID(3,0), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) },
@@ -289,6 +299,8 @@ static struct {
 };
 static const int	nbufsz = sizeof(bufinst) / sizeof(bufinst[0]);
 
+static char *zoneinfo = NULL;
+
 /*
  * Per-context structures
  */
@@ -1111,6 +1123,175 @@ tzinfo(void)
     return __pmTimezone();
 }
 
+#define ZONEINFO "/zoneinfo/"
+#define LOCALTIME "/etc/localtime"
+
+/*
+ * Plan B
+ *
+ * Get the size for /etc/localtime if possible.
+ *
+ * Descend /usr/share/zoneinfo looking for a file with the same size
+ * and then compare file contents ... if equal, consider it a match.
+ *
+ * If more than one file matches, pick the first but emit a warning.
+ */
+char *
+getzoneinfo_plan_b(void)
+{
+    FILE	*fp;		/* find ... pipe */
+    FILE	*f1;		/* /etc/localtime */
+    FILE	*f2;		/* candidate file */
+    int		c1;
+    int		c2;
+    char	*p;
+    char	*path = NULL;
+    struct stat	sbuf;		/* sbuf.st_size is all that matters */
+    char	tmp[MAXPATHLEN+2];	/* shell command and line buffer */
+
+    if ((f1 = fopen(LOCALTIME, "r")) == NULL) {
+	fprintf(stderr, "getzoneinfo_plan_b: cannot open %s: %s\n", LOCALTIME, pmErrStr(-oserror()));
+	return NULL;
+    }
+
+    if (fstat(fileno(f1), &sbuf) < 0) {
+	fprintf(stderr, "getzoneinfo_plan_b: cannot stat %s: %s\n", LOCALTIME, pmErrStr(-oserror()));
+	fclose(f1);
+	return NULL;
+    }
+    sprintf(tmp, "find /usr/share/zoneinfo -type f -a -size %ldc", sbuf.st_size);
+    if ((fp = popen(tmp, "r")) == NULL) {
+	fprintf(stderr, "getzoneinfo_plan_b: pipe(%s) failed: %s\n", tmp, pmErrStr(-oserror()));
+	fclose(f1);
+	return NULL;
+    }
+    /* start at reading at tmp[1], so ':' can be inserted at tmp[0] */
+    while (fgets(&tmp[1], sizeof(tmp)-1, fp) != NULL) {
+	/* strip \n at end of line */
+	for (p = &tmp[1]; *p != '\n'; p++)
+	    ;
+	*p = '\0';
+	if ((f2 = fopen(&tmp[1], "r")) == NULL) {
+	    fprintf(stderr, "getzoneinfo_plan_b: cannot open %s: %s\n", &tmp[1], pmErrStr(-oserror()));
+	    fclose(f1);
+	    pclose(fp);
+	    if (path != NULL)
+		free(path);
+	    return NULL;
+	}
+	rewind(f1);
+
+	for ( ; ; ) {
+	    c1 = fgetc(f1);
+	    c2 = fgetc(f2);
+	    if (c1 == EOF && c2 == EOF) {
+		/* contents match */
+		if (path == NULL) {
+		    tmp[0] = ':';
+		    path = strdup(tmp);
+		    if (path == NULL) {
+			fprintf(stderr, "getzoneinfo_plan_b: match %s but strdup failed\n", &tmp[1]);
+			fclose(f2);
+			fclose(f1);
+			pclose(fp);
+			return NULL;
+		    }
+		}
+		else {
+		    /*
+		     * Duplicates ... pick shortest path, as this will favour, for
+		     * example, Australia/Melbourne over posix/Australia/Melbourne
+		     */
+		    if (strlen(&path[1]) <= strlen(&tmp[1]))
+			fprintf(stderr, "getzoneinfo_plan_b: Warning: match %s and %s, choosing first one\n", &path[1], &tmp[1]);
+		    else {
+			fprintf(stderr, "getzoneinfo_plan_b: Warning: match %s and %s, choosing second one\n", &path[1], &tmp[1]);
+			free(path);
+			tmp[0] = ':';
+			path = strdup(tmp);
+			if (path == NULL) {
+			    fprintf(stderr, "getzoneinfo_plan_b: strdup failed\n");
+			    fclose(f2);
+			    fclose(f1);
+			    pclose(fp);
+			    return NULL;
+			}
+		    }
+		}
+		break;
+	    }
+	    if (c1 != c2)
+		break;
+	}
+	fclose(f2);
+    }
+    fclose(f1);
+    pclose(fp);
+
+    return path;
+}
+
+/*
+ * Get the local timezone tzfile identification
+ *
+ * We'd like this to return something like ":Australia/Melbourne"
+ * that can be used as a $TZ setting.
+ *
+ * Plan A
+ *   If /etc/localtime is a symbolic link, get the pathname it points
+ *   to and strip anything up to (and including) the string "/zoneinfo/".
+ *
+ * Return NULL on failure.
+ */
+char *
+getzoneinfo(void)
+{
+    ssize_t	sts;
+    char	*buf;
+    char	*tmp_buf;
+    char	*p;
+    char	*q;
+
+    /* +1 for : +1 for NULL */
+    buf = (char *)malloc(MAXPATHLEN+2);
+    if (buf == NULL)
+	return NULL;
+
+    sts = readlink(LOCALTIME, &buf[1], MAXPATHLEN);
+    if (sts < 0) {
+	/*
+	 * Hmm, not a symlink. Now try Plan B.
+	 */
+	free(buf);
+	buf = getzoneinfo_plan_b();
+	if (buf == NULL)
+	    return NULL;
+    }
+    else
+	buf[sts+1] = '\0';
+
+    /* try to find prefix .../zoneinfo/... */
+    p = strstr(buf, ZONEINFO);
+    if (p != NULL) {
+	/* found it! */
+	q = &p[strlen(ZONEINFO)-1];
+	tmp_buf = strdup(q);
+	if (tmp_buf != NULL) {
+	    free(buf);
+	    buf = tmp_buf;
+	}
+    }
+    else {
+	/* no prefix, truncate ... nice to have, not necessary */
+	tmp_buf = realloc(buf, sts+2);
+	if (tmp_buf != NULL)
+	    buf = tmp_buf;
+    }
+    buf[0] = ':';
+
+    return buf;
+}
+
 static char *
 services(void)
 {
@@ -1227,10 +1408,9 @@ fetch_labels(int ctx, pmAtomValue *avp, char **hostname)
 	avp->cp = pmcd_labels;
     else if ((sts = __pmGetContextLabels(&set)) <= 0)
 	avp->cp = "";
-    else {
+    else
 	avp->cp = strndup(set->json, set->jsonlen + 1);
-	pmFreeLabelSets(set, 1);
-    }
+    pmFreeLabelSets(set, 1);
     return avp->cp;
 }
 
@@ -1533,6 +1713,17 @@ pmcd_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 				fetch_labels(pmda->e_context, &atom, &host);
 				break;
 
+			case 26:	/* zoneinfo */
+				if (zoneinfo == NULL) {
+				    zoneinfo = getzoneinfo();
+				    if (zoneinfo == NULL) {
+					/* value is an empty string if not available */
+					zoneinfo = "";
+				    }
+				}
+				atom.cp = zoneinfo;
+				break;
+
 			default:
 				sts = atom.l = PM_ERR_PMID;
 				break;
