@@ -20,8 +20,9 @@
 #include <float.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include "../libpcp/src/internal.h"
 
-static struct pmTimeval	pmtv;
+static struct timeval	tv;
 static char		timebuf[32];	/* for pmCtime result + .xxx */
 static int		numpmid;
 static pmID		*pmid;
@@ -38,6 +39,7 @@ static pmLongOptions longopts[] = {
     { "descs", 0, 'd', 0, "dump metric descriptions" },
     { "labelsets", 0, 'e', 0, "dump all metric label metadata" },
     { "helptext", 0, 'h', 0, "dump all metric help text" },
+    { "on-disk-insts", 0, 'I', 0, "dump on-disk instance domain descriptions" },
     { "insts", 0, 'i', 0, "dump instance domain descriptions" },
     { "", 0, 'L', 0, "more verbose form of archive label dump" },
     { "label", 0, 'l', 0, "dump the archive label" },
@@ -61,7 +63,7 @@ static pmLongOptions longopts[] = {
 static int overrides(int, pmOptions *);
 static pmOptions opts = {
     .flags = PM_OPTFLAG_DONE | PM_OPTFLAG_STDOUT_TZ | PM_OPTFLAG_BOUNDARIES,
-    .short_options = "aD:dehilLmMn:rS:sT:tv:xZ:z?",
+    .short_options = "aD:dehIilLmMn:rS:sT:tv:xZ:z?",
     .long_options = longopts,
     .short_usage = "[options] [archive [metricname ...]]",
     .override = overrides,
@@ -85,6 +87,34 @@ tvcmp(struct timeval a, struct timeval b)
     return 0;
 }
 
+static void
+mytimestamp(struct timeval *stamp)
+{
+    struct pmTimeval	pmtv;
+
+    pmtv.tv_sec = stamp->tv_sec;
+    pmtv.tv_usec = stamp->tv_usec;
+
+    if (xflag) {
+	char	       	*ddmm;
+	char	       	*yr;
+	time_t		time;
+
+	time = stamp->tv_sec;
+	ddmm = pmCtime(&time, timebuf);
+	ddmm[10] = '\0';
+	yr = &ddmm[20];
+	printf("%s ", ddmm);
+	__pmPrintTimeval(stdout, &pmtv);
+	printf(" %4.4s", yr);
+	if (xflag >= 2)
+	    printf(" (%.6f)", pmtimevalSub(stamp, &label.ll_start));
+    }
+    else {
+	__pmPrintTimeval(stdout, &pmtv);
+    }
+}
+
 static int
 do_size(pmResult *rp)
 {
@@ -134,8 +164,8 @@ setup_event_derived_metrics(void)
 	 * __pmRegisterAnon(), so the anon metrics
 	 * should now be in the PMNS
 	 */
-	char	*name_flags = "event.flags";
-	char	*name_missed = "event.missed";
+	const char	*name_flags = "event.flags";
+	const char	*name_missed = "event.missed";
 
 	if ((sts = pmLookupName(1, &name_flags, &pmid_flags)) < 0) {
 	    /* should not happen! */
@@ -309,9 +339,7 @@ dump_event(int numnames, char **names, pmValueSet *vsp, int index, int indom, in
 
 	for (r = 0; r < nrecords; r++) {
 	    printf("        --- event record [%d] timestamp ", r);
-	    pmtv.tv_sec = res[r]->timestamp.tv_sec;
-	    pmtv.tv_usec = res[r]->timestamp.tv_usec;
-	    __pmPrintTimeval(stdout, &pmtv);
+	    mytimestamp(&res[r]->timestamp);
 	    if (dump_nparams(res[r]->numpmid) < 0)
 		continue;
 	    flags = 0;
@@ -369,28 +397,7 @@ dump_result(pmResult *resp)
 	printf("[%d bytes]\n", nbyte);
     }
 
-    if (xflag) {
-	char	       *ddmm;
-	char	       *yr;
-	time_t		time;
-
-	time = resp->timestamp.tv_sec;
-	ddmm = pmCtime(&time, timebuf);
-	ddmm[10] = '\0';
-	yr = &ddmm[20];
-	printf("%s ", ddmm);
-	pmtv.tv_sec = resp->timestamp.tv_sec;
-	pmtv.tv_usec = resp->timestamp.tv_usec;
-	__pmPrintTimeval(stdout, &pmtv);
-	printf(" %4.4s", yr);
-	if (xflag >= 2)
-	    printf(" (%.6f)", pmtimevalSub(&resp->timestamp, &label.ll_start));
-    }
-    else {
-	pmtv.tv_sec = resp->timestamp.tv_sec;
-	pmtv.tv_usec = resp->timestamp.tv_usec;
-	__pmPrintTimeval(stdout, &pmtv);
-    }
+    mytimestamp(&resp->timestamp);
 
     if (resp->numpmid == 0) {
 	printf("  <mark>\n");
@@ -469,6 +476,100 @@ dumpDesc(__pmContext *ctxp)
     }
 }
 
+static void
+dumpDiskInDom(__pmContext *ctxp)
+{
+    size_t	n;
+    size_t	rlen;
+    __int32_t	check;
+    __pmLogHdr	hdr;
+    __pmFILE	*f = ctxp->c_archctl->ac_log->l_mdfp;
+
+    printf("\nInstance Domains on-disk ...\n");
+
+    __pmFseek(f, (long)(sizeof(__pmLogLabel) + 2*sizeof(int)), SEEK_SET);
+    for ( ; ; ) {
+	n = __pmFread(&hdr, 1, sizeof(__pmLogHdr), f);
+	hdr.len = ntohl(hdr.len);
+	hdr.type = ntohl(hdr.type);
+	if (n != sizeof(__pmLogHdr)) {
+            if (__pmFeof(f)) {
+		/* end-of-file, all done. */
+                return;
+            }
+	    if (__pmFerror(f)) {
+		fprintf(stderr, "dumpDiskInDom: hdr __pmFread(%zd) -> %s\n", sizeof(__pmLogHdr), pmErrStr(-oserror())); 
+		return;
+	    }
+	    fprintf(stderr, "dumpDiskInDom: Botch: hdr __pmFread(%zd) -> %zd\n", sizeof(__pmLogHdr), n);
+	    return;
+	}
+	if (hdr.len <= 0) {
+	    fprintf(stderr, "dumpDiskInDom: Botch: hdr.len < 0 (%d)\n", hdr.len);
+	    return;
+	}
+	rlen = (size_t)hdr.len - sizeof(__pmLogHdr) - sizeof(__int32_t);
+	if (hdr.type == TYPE_INDOM) {
+	    pmTimeval		*tvp;
+	    pmInResult		in;
+	    char		*namebase;
+	    __int32_t		*rbuf;
+	    __int32_t		*stridx;
+	    int			i;
+	    int			k;
+
+	    if ((rbuf = (__int32_t *)malloc(rlen)) == NULL) {
+		fprintf(stderr, "dumpDiskInDom: tbuf: malloc failed: %zd\n", rlen);
+		exit(1);
+	    }
+	    if ((n = __pmFread(rbuf, 1, rlen, f)) != rlen) {
+		fprintf(stderr, "dumpDiskInDom: indom __pmFread(%zd) -> %zd\n",
+			    rlen, n);
+		free(rbuf);
+		return;
+	    }
+
+	    k = 0;
+	    tvp = (pmTimeval *)&rbuf[k];
+	    tv.tv_sec = ntohl(tvp->tv_sec);
+	    tv.tv_usec = ntohl(tvp->tv_usec);
+	    k += sizeof(pmTimeval)/sizeof(int);
+	    in.indom = __ntohpmInDom((unsigned int)rbuf[k++]);
+	    in.numinst = ntohl(rbuf[k++]);
+	    mytimestamp(&tv);
+	    printf(" InDom: %s", pmInDomStr(in.indom));
+	    printf(" %d instances\n", in.numinst);
+	    if (in.numinst > 0) {
+		in.instlist = &rbuf[k];
+		k += in.numinst;
+		stridx = &rbuf[k];
+		k += in.numinst;
+		namebase = (char *)&rbuf[k];
+	        for (i = 0; i < in.numinst; i++) {
+		    printf("   %d or \"%s\"\n",
+			ntohl(in.instlist[i]), &namebase[ntohl(stridx[i])]);
+		}
+	    }
+	    free(rbuf);
+	}
+	else {
+	    /* skip this record, not TYPE_INDOM */
+	    __pmFseek(f, (off_t)rlen, SEEK_CUR);
+	}
+	/* trailer check */
+	n = __pmFread(&check, 1, sizeof(check), f);
+	if (n != sizeof(check)) {
+	    fprintf(stderr, "dumpDiskInDom: Botch: trailer __pmFread(%zd) -> %zd\n", sizeof(__pmLogHdr), n);
+	    return;
+	}
+	check = ntohl(check);
+	if (hdr.len != check) {
+	    fprintf(stderr, "dumpDiskInDom: Botch: trailer len (%d) != hdr len (%d)\n", check, hdr.len);
+	    return;
+	}
+    }
+}
+
 static void
 dumpInDom(__pmContext *ctxp)
 {
@@ -488,7 +589,9 @@ dumpInDom(__pmContext *ctxp)
 	    for ( ; ; ) {
 		for (idp = (__pmLogInDom *)hp->data; idp->next != ldp; idp =idp->next)
 			;
-		__pmPrintTimeval(stdout, &idp->stamp);
+		tv.tv_sec = idp->stamp.tv_sec;
+		tv.tv_usec = idp->stamp.tv_usec;
+		mytimestamp(&tv);
 		printf(" %d instances\n", idp->numinst);
 		for (j = 0; j < idp->numinst; j++) {
 		    printf("   %d or \"%s\"\n",
@@ -662,7 +765,9 @@ dumpLabelSets(__pmContext *ctxp)
 	 * Now print all the label sets at this time stamp.
 	 * Sort by type and then identifier.
 	 */
-	__pmPrintTimeval(stdout, &this_stamp);
+	tv.tv_sec = this_stamp.tv_sec;
+	tv.tv_usec = this_stamp.tv_usec;
+	mytimestamp(&tv);
 	putchar('\n');
 	for (lix = 0; lix < sizeof(labelTypes) / sizeof(*labelTypes); ++lix) {
 	    /* Are there labels of this type? */
@@ -744,11 +849,15 @@ dumpTI(__pmContext *ctxp)
     lcp = ctxp->c_archctl->ac_log;
 
     printf("\nTemporal Index\n");
+    if (xflag)
+	printf("\t\t");
     printf("\t\tLog Vol    end(meta)     end(log)\n");
     lastp = NULL;
     for (i = 0; i < ctxp->c_archctl->ac_log->l_numti; i++) {
 	tip = &ctxp->c_archctl->ac_log->l_ti[i];
-	__pmPrintTimeval(stdout, &tip->ti_stamp);
+	tv.tv_sec = tip->ti_stamp.tv_sec;
+	tv.tv_usec = tip->ti_stamp.tv_usec;
+	mytimestamp(&tv);
 	printf("\t  %5d  %11d  %11d\n", tip->ti_vol, tip->ti_meta, tip->ti_log);
 	if (i == 0) {
 	    pmsprintf(path, sizeof(path), "%s.meta", lcp->l_name);
@@ -868,7 +977,8 @@ rawdump(FILE *f)
 	return;
     }
 
-    fseek(f, (long)0, SEEK_SET);
+    if (fseek(f, (long)0, SEEK_SET) < 0)
+	fprintf(stderr, "Warning: fseek(..., 0, ...) failed: %s\n", pmErrStr(-oserror()));
 
     while ((sts = fread(&len, 1, sizeof(len), f)) == sizeof(len)) {
 	len = ntohl(len);
@@ -899,7 +1009,8 @@ rawdump(FILE *f)
     }
     if (sts < 0)
 	printf("fread fails: %s\n", osstrerror());
-    fseek(f, old, SEEK_SET);
+    if (fseek(f, old, SEEK_SET) < 0)
+	fprintf(stderr, "Warning: fseek(..., %ld, ...) failed: %s\n", old, pmErrStr(-oserror()));
 }
 
 static void
@@ -913,7 +1024,7 @@ dometric(const char *name)
     }
     numpmid++;
     pmid = (pmID *)realloc(pmid, numpmid * sizeof(pmID));
-    if ((sts = pmLookupName(1, (char **)&name, &pmid[numpmid-1])) < 0) {
+    if ((sts = pmLookupName(1, &name, &pmid[numpmid-1])) < 0) {
 	fprintf(stderr, "%s: pmLookupName(%s): %s\n", pmGetProgname(), name, pmErrStr(sts));
 	numpmid--;
     }
@@ -958,6 +1069,7 @@ main(int argc, char *argv[])
     int			dflag = 0;
     int			eflag = 0;
     int			hflag = 0;
+    int			Iflag = 0;
     int			iflag = 0;
     int			Lflag = 0;
     int			lflag = 0;
@@ -991,6 +1103,10 @@ main(int argc, char *argv[])
 	    hflag = 1;
 	    break;
 
+	case 'I':	/* dump on-disk instance domains */
+	    Iflag = 1;
+	    break;
+
 	case 'i':	/* dump instance domains */
 	    iflag = 1;
 	    break;
@@ -1030,7 +1146,7 @@ main(int argc, char *argv[])
 	    break;
 
 	case 'x':	/* report Ddd Mmm DD <timestamp> YYYY */
-			/* -xx reports numeric timeval also */
+			/* -xx reports time offset from start of archive also */
 	    xflag++;
 	    break;
 	}
@@ -1056,7 +1172,7 @@ main(int argc, char *argv[])
 	exit(0);
     }
 
-    if (dflag + eflag + hflag + iflag + lflag + mflag + tflag == 0)
+    if (dflag + eflag + hflag + Iflag + iflag + lflag + mflag + tflag == 0)
 	mflag = 1;	/* default */
 
     /* delay option end processing until now that we have the archive name */
@@ -1088,9 +1204,10 @@ main(int argc, char *argv[])
 	numpmid = 0;
 	pmid = NULL;
 	for (i = 0; opts.optind < argc; i++, opts.optind++) {
+	    const char *name = argv[opts.optind];
 	    numpmid++;
 	    pmid = (pmID *)realloc(pmid, numpmid * sizeof(pmID));
-	    if ((sts = pmLookupName(1, &argv[opts.optind], &pmid[numpmid-1])) < 0) {
+	    if ((sts = pmLookupName(1, &name, &pmid[numpmid-1])) < 0) {
 		if (sts == PM_ERR_NONLEAF) {
 		    numpmid--;
 		    if ((sts = pmTraversePMNS(argv[opts.optind], dometric)) < 0)
@@ -1143,9 +1260,13 @@ main(int argc, char *argv[])
     PM_UNLOCK(ctxp->c_lock);
 
     if (mode == PM_MODE_FORW)
-	pmSetMode(mode, &opts.start, 0);
+	sts = pmSetMode(mode, &opts.start, 0);
     else
-	pmSetMode(mode, &opts.finish, 0);
+	sts = pmSetMode(mode, &opts.finish, 0);
+    if (sts < 0) {
+	fprintf(stderr, "%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
+	exit(1);
+    }
 
     if (lflag)
 	dumpLabel(Lflag);
@@ -1159,6 +1280,9 @@ main(int argc, char *argv[])
     if (hflag)
 	dumpHelpText(ctxp);
 
+    if (Iflag)
+	dumpDiskInDom(ctxp);
+
     if (iflag)
 	dumpInDom(ctxp);
 
@@ -1188,10 +1312,6 @@ main(int argc, char *argv[])
 		done.tv_usec = 0;
 	    }
 	}
-	if (sts < 0) {
-	    fprintf(stderr, "%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
-	    exit(1);
-	}
 	sts = 0;
 	for ( ; ; ) {
 	    sts = __pmLogFetch(ctxp, 0, NULL, &raw_result);
