@@ -27,6 +27,7 @@ myLocalFetch(__pmContext *ctxp, int numpmid, pmID pmidlist[], __pmPDU **pdup)
     pmResult	*result;
     pmID	*newlist = NULL;
     int		newcnt, have_dm, n;
+    int		sts;
 
     /* for derived metrics, may need to rewrite the pmidlist */
     have_dm = newcnt = __pmPrepareFetch(ctxp, numpmid, pmidlist, &newlist);
@@ -49,7 +50,10 @@ myLocalFetch(__pmContext *ctxp, int numpmid, pmID pmidlist[], __pmPDU **pdup)
 	    free(newlist);
     }
 
-    return __pmEncodeResult(0, result, pdup);
+    sts = __pmEncodeResult(0, result, pdup);
+    pmFreeResult(result);
+
+    return sts;
 }
 
 /*
@@ -248,8 +252,10 @@ myFetch(int numpmid, pmID pmidlist[], __pmPDU **pdup)
 			}
 			else {
 			    __pmFinishResult(ctxp, sts, &result);
-			    if ((sts = __pmEncodeResult(ctxp->c_pmcd->pc_fd, result, &npb)) < 0)
+			    if ((sts = __pmEncodeResult(ctxp->c_pmcd->pc_fd, result, &npb)) < 0) {
+				pmFreeResult(result);
 				n = sts;
+			    }
 			    else {
 				/* using PDU with derived metrics */
 				__pmUnpinPDUBuf(pb);
@@ -271,12 +277,14 @@ myFetch(int numpmid, pmID pmidlist[], __pmPDU **pdup)
 		    else {
 			fprintf(stderr, "myFetch: ERROR PDU: %s\n", pmErrStr(n));
 			disconnect(PM_ERR_IPC);
+			changed = 0;
 		    }
 		    __pmUnpinPDUBuf(pb);
 		}
 		else if (n == 0) {
 		    fprintf(stderr, "myFetch: End of File: PMCD exited?\n");
 		    disconnect(PM_ERR_IPC);
+		    changed = 0;
 		}
 		else if (n == -EINTR) {
 		    /* SIGINT, let the normal cleanup happen */
@@ -286,11 +294,13 @@ myFetch(int numpmid, pmID pmidlist[], __pmPDU **pdup)
 		    /* other badness, disconnect */
 		    fprintf(stderr, "myFetch: __pmGetPDU: Error: %s\n", pmErrStr(n));
 		    disconnect(PM_ERR_IPC);
+		    changed = 0;
 		}
 		else {
 		    /* protocol botch, disconnect */
 		    fprintf(stderr, "myFetch: Unexpected %s PDU from PMCD\n", __pmPDUTypeStr(n));
 		    disconnect(PM_ERR_IPC);
+		    changed = 0;
 		    __pmUnpinPDUBuf(pb);
 		}
 	    } while (n == 0);
