@@ -11,6 +11,16 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * Debugging options
+ * appl0	control request pdus (via pmlc)
+ * appl1	credentials exchange
+ * appl2	alarm (timing) loop
+ * 		callback (do_work())
+ * 		record mode control messages
+ * appl3	signal callbacks and exit logging
+ * appl4	desperate tracing to help triage qa/793 failures
+ * 		[THIS IS TEMPORARY AND WILL BE REMOVED]
  */
 
 #include <ctype.h>
@@ -75,6 +85,9 @@ run_done(int sts, char *msg)
 {
     int	i, lsts;
 
+    /* no more timer events, especially on the re-exec path */
+    __pmAFblock();
+
     if (pmDebugOptions.services || (pmDebugOptions.log && pmDebugOptions.desperate)) {
 	fprintf(stderr, "run_done(%d, %s) last_log_offset=%d last_stamp=",
 		sts, msg, last_log_offset);
@@ -477,7 +490,6 @@ do_dialog(char cmd)
 	    fprintf(stderr, "Reason? %s\n", osstrerror());
 	    if (fd != -1)
 		close(fd);
-	    free(xconfirm);
 	    goto failed;
 	}
 	fputs(p, msgf);
@@ -486,11 +498,11 @@ do_dialog(char cmd)
 
 	if (cmd == 'X')
 	    pmsprintf(lbuf, sizeof(lbuf), "%s -c -header \"%s - %s\" -file %s -icon question "
-			  "-B Yes -b No 2>/dev/null",
+			  "-B Yes -b No",
 		    xconfirm, dialog_title, rsc_prog, msg);
 	else
 	    pmsprintf(lbuf, sizeof(lbuf), "%s -c -header \"%s - %s\" -file %s -icon info "
-			  "-b Close 2>/dev/null",
+			  "-b Close",
 		    xconfirm, dialog_title, rsc_prog, msg);
 
 
@@ -498,13 +510,11 @@ do_dialog(char cmd)
 	    fprintf(stderr, "\nError: __pmProcessUnpickArgs failed for recording session dialog\n");
 	    fprintf(stderr, "Command: \"%s\"\n", lbuf);
 	    fprintf(stderr, "Error: %s\n", pmErrStr(sts));
-	    free(xconfirm);
 	    goto failed;
 	}
 	if ((sts = __pmProcessPipe(&argp, "r", PM_EXEC_TOSS_NONE, &msgf)) < 0) {
 	    fprintf(stderr, "\nError: failed to start command for recording session dialog\n");
 	    fprintf(stderr, "Command: \"%s\"\n", lbuf);
-	    free(xconfirm);
 	    goto failed;
 	}
 
@@ -715,12 +725,12 @@ updateLatestFolio(const char *host, const char *base)
      */
     if (getcwd(dir, sizeof(dir)) == NULL) {
 	if (pmDebugOptions.services)
-	    fprintf(stderr, "Info: updateLatestFolio: getcwd() failed for host %s: %s", host, strerror(errno));
+	    fprintf(stderr, "Info: updateLatestFolio: getcwd() failed for host %s: %s\n", host, strerror(errno));
 	return;
     }
     if (strncmp(dir, logdir, strlen(logdir)) != 0) {
 	if (pmDebugOptions.services)
-	    fprintf(stderr, "Info: not creating \"Latest\" archive folio for host %s: %s", host, strerror(errno));
+	    fprintf(stderr, "Info: not creating \"Latest\" archive folio for host %s: cwd %s not below %s\n", host, dir, logdir);
     	return;
     }
 
@@ -728,7 +738,7 @@ updateLatestFolio(const char *host, const char *base)
     thishost[MAXHOSTNAMELEN-1] = '\0';
 
     if ((fp = fopen("Latest", "w")) == NULL) {
-    	fprintf(stderr, "Warning: failed to create \"Latest\" archive folio for host %s: %s", host, strerror(errno));
+    	fprintf(stderr, "Warning: failed to create \"Latest\" archive folio for host %s: %s\n", host, strerror(errno));
 	return;
     }
     time(&now);
@@ -1033,7 +1043,7 @@ main(int argc, char **argv)
 	/* only open a new log if we are NOT reexec'd */
 	if (pmlogger_reexec) {
 	    if (pmDebugOptions.services)
-		fprintf(stderr, "existing pmlogger.log remains open after re-exec\n");
+		fprintf(stderr, "existing log %s remains open after re-exec\n", logfile);
 	} else {
 	    pmOpenLog("pmlogger", logfile, stderr, &sts);
 	    if (sts != 1) {
@@ -1098,10 +1108,12 @@ main(int argc, char **argv)
     else if (pmcd_host_conn == NULL)
 	pmcd_host_conn = "local:";
 
+    if (pmDebugOptions.appl4) fprintf(stderr, "@pmNewContext\n");
     if ((ctx = pmNewContext(host_context, pmcd_host_conn)) < 0) {
 	fprintf(stderr, "%s: Cannot connect to PMCD on host \"%s\": %s\n", pmGetProgname(), pmcd_host_conn, pmErrStr(ctx));
 	exit(1);
     }
+    if (pmDebugOptions.appl4) fprintf(stderr, "@pmGetContextHostName\n");
     pmcd_host = (char *)pmGetContextHostName(ctx);
     if (strlen(pmcd_host) == 0) {
 	fprintf(stderr, "%s: pmGetContextHostName(%d) failed\n",
@@ -1111,6 +1123,7 @@ main(int argc, char **argv)
 
     if (rsc_fd == -1 && host_context != PM_CONTEXT_LOCAL) {
 	/* no -x, so register client id with pmcd */
+	if (pmDebugOptions.appl4) fprintf(stderr, "@__pmSetClientIdArgv\n");
 	__pmSetClientIdArgv(argc, argv);
     }
 
@@ -1118,6 +1131,7 @@ main(int argc, char **argv)
      * discover fd for comms channel to PMCD ... 
      */
     if (host_context != PM_CONTEXT_LOCAL) {
+	if (pmDebugOptions.appl4) fprintf(stderr, "@__pmHandleToPtr\n");
 	if ((ctxp = __pmHandleToPtr(ctx)) == NULL) {
 	    fprintf(stderr, "%s: botch: __pmHandleToPtr(%d) returns NULL!\n", pmGetProgname(), ctx);
 	    exit(1);
@@ -1126,6 +1140,7 @@ main(int argc, char **argv)
 	PM_UNLOCK(ctxp->c_lock);
     }
 
+    if (pmDebugOptions.appl4) fprintf(stderr, "@do_pmcpp\n");
     yyin = do_pmcpp(configfile);
     /* do not return unless yyin is valid */
     if (configfile == NULL)
